# -*- coding: utf-8 -*-
r"""
Some fruits

This file is an example of Sage conventions for syntax and documentation.
It also serves as an example to explain class inheritance: methods defined
for fruits are automatically defined for bananas and strawberries.

AUTHORS:

- Sébastien Labbé, 2010-2013

EXAMPLES::

    sage: from slabbe import Fruit
    sage: f = Fruit(5)
    sage: f
    A fruit of 5 kilos.
    sage: f.weight()
    5
    sage: f.is_a_fruit()
    True

Because of class inheritance which says that a banana is a fruit and a
strawberry is a fruit, the methods written for fruits are defined for
bananas and strawberries automatically::

    sage: from slabbe import Strawberry
    sage: s = Strawberry(32)
    sage: s
    A strawberry of 32 kilos.
    sage: s.weight()
    32
    sage: s.is_a_fruit()
    True

::

    sage: from slabbe import Banana
    sage: b = Banana(13)
    sage: b
    A banana of 13 kilos.
    sage: b.weight()
    13
    sage: b.is_a_fruit()
    True

::

    sage: s = Strawberry(32)
    sage: t = Strawberry(7)
    sage: s
    A strawberry of 32 kilos.
    sage: t
    A strawberry of 7 kilos.
    sage: s + t
    A strawberry of 1073 kilos.

"""
#*****************************************************************************
#       Copyright (C) 2010-2013 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from sage.structure.sage_object import SageObject

class Fruit(SageObject):
    r"""
    Creates a fruit.

    INPUT:

    - ``weight`` - number, in kilos

    OUTPUT:

        Fruit

    EXAMPLES::

        sage: from slabbe import Fruit
        sage: f = Fruit(5)
        sage: f
        A fruit of 5 kilos.

    """
    def __init__(self, weight=1):
        r"""
        Constructor.

        EXAMPLES::

            sage: from slabbe import Fruit
            sage: f = Fruit(3)

        TESTS:

        Testing that pickle works::

            sage: loads(dumps(f))
            A fruit of 3 kilos.
            sage: f == loads(dumps(f))
            True

        Running the test suite::

            sage: TestSuite(f).run()
        """
        self._weight = weight

    def _repr_(self):
        r"""
        Returns the string representation.

        OUTPUT:

            String

        EXAMPLES::

            sage: from slabbe import Fruit
            sage: f = Fruit(3)
            sage: f
            A fruit of 3 kilos.
        """
        return "A fruit of %s kilos."%self._weight

    def __eq__(self, other):
        r"""
        Return if self is equal to other.

        INPUT:

        - ``other`` - an object

        OUTPUT:

            Boolean

        EXAMPLES::

            sage: from slabbe import Fruit
            sage: s = Fruit(32)
            sage: t = Fruit(7)
            sage: u = Fruit(32)
            sage: s == u
            True
            sage: s == t
            False
        """
        return (isinstance(other, Fruit) and
                self.weight() == other.weight())
    def weight(self):
        r"""
        Return the weight.

        OUTPUT:

            Number

        EXAMPLES::

            sage: from slabbe import Fruit
            sage: f = Fruit(3)
            sage: f.weight()
            3
        """
        return self._weight

    def is_a_fruit(self):
        r"""
        Returns True if it is a fruit.

        OUTPUT:

            Boolean

        EXAMPLES::

            sage: from slabbe import Fruit
            sage: f = Fruit(3)
            sage: f.is_a_fruit()
            True
        """
        return True

class Banana(Fruit):
    r"""
    Creates a banana.

    INPUT:

    - ``weight`` - number, in kilos

    OUTPUT:

        Banana

    EXAMPLES::

        sage: from slabbe import Banana
        sage: b = Banana(9)
        sage: b
        A banana of 9 kilos.

    TESTS:

    Testing that pickle works::

        sage: loads(dumps(b))
        A banana of 9 kilos.
        sage: b == loads(dumps(b))
        True

    Running the test suite::

        sage: TestSuite(b).run()
    """
    def _repr_(self):
        r"""
        Returns the string representation.

        OUTPUT:

            String

        EXAMPLES::

            sage: from slabbe import Banana
            sage: b = Banana(9)
            sage: b
            A banana of 9 kilos.
        """
        return "A banana of %s kilos."%self._weight

    def __eq__(self, other):
        r"""
        Return if self is equal to other.

        INPUT:

        - ``other`` - an object

        OUTPUT:

            Boolean

        EXAMPLES::

            sage: from slabbe import Banana
            sage: s = Banana(32)
            sage: t = Banana(7)
            sage: u = Banana(32)
            sage: s == u
            True
            sage: s == t
            False
        """
        return (isinstance(other, Banana) and
                self.weight() == other.weight())
class Strawberry(Fruit):
    r"""
    Creates a strawberry.

    INPUT:

    - ``weight`` - number, in kilos

    OUTPUT:

        Strawberry

    EXAMPLES::

        sage: from slabbe import Strawberry
        sage: s = Strawberry(34)
        sage: s
        A strawberry of 34 kilos.

    TESTS:

    Testing that pickle works::

        sage: loads(dumps(s))
        A strawberry of 34 kilos.
        sage: s == loads(dumps(s))
        True

    Running the test suite::

        sage: TestSuite(s).run()
    """
    def _repr_(self):
        r"""
        Returns the string representation.

        OUTPUT:

            String

        EXAMPLES::

            sage: from slabbe import Strawberry
            sage: f = Strawberry(34)
            sage: f
            A strawberry of 34 kilos.
        """
        return "A strawberry of %s kilos."%self._weight

    def __eq__(self, other):
        r"""
        Return if self is equal to other.

        INPUT:

        - ``other`` - an object

        OUTPUT:

            Boolean

        EXAMPLES::

            sage: from slabbe import Strawberry
            sage: s = Strawberry(32)
            sage: t = Strawberry(7)
            sage: u = Strawberry(32)
            sage: s == u
            True
            sage: s == t
            False
        """
        return (isinstance(other, Strawberry) and
                self.weight() == other.weight())
    def __add__(self, other):
        r"""
        Fusion of a strawberry with another strawberry.

        INPUT:

        - ``other`` - a strawberry

        OUTPUT:

            Strawberry

        EXAMPLES::

            sage: from slabbe import Strawberry
            sage: f = Strawberry(34)
            sage: g = Strawberry(20)
            sage: f + g
            A strawberry of 1556 kilos.
        """
        return Strawberry(self.weight()**2 + other.weight()**2)




# -*- coding: utf-8 -*-
r"""
Simultaneous diophantine approximation

EXAMPLES::

    sage: from slabbe.diophantine_approx import best_simultaneous_convergents
    sage: it = best_simultaneous_convergents([e, pi])
    sage: dirichlet10 = [next(it) for _ in range(10)]
    sage: dirichlet10
    [(3, 3, 1),
     (19, 22, 7),
     (1843, 2130, 678),
     (51892, 59973, 19090),
     (113018, 130618, 41577),
     (114861, 132748, 42255),
     (166753, 192721, 61345),
     (446524, 516060, 164267),
     (1174662, 1357589, 432134),
     (3970510, 4588827, 1460669)]

The above Dirichlet simultaneous diophantine approximations appear as columns of
matrices generated by multidimensional continued fraction algorithms::

    sage: from slabbe.diophantine_approx import mult_cont_frac_vs_dirichlet
    sage: from slabbe.mult_cont_frac import Brun,ARP,Reverse,Selmer,Cassaigne
    sage: algos = [Brun(), ARP(), Reverse(), Selmer(),Cassaigne()]
    sage: mult_cont_frac_vs_dirichlet([e,pi], dirichlet10, algos)
      Dirichlet                     Brun       ARP        Reverse   Selmer     Cassaigne
    +-----------------------------+----------+----------+---------+----------+-----------+
      (3, 3, 1)                     [4, 5]     [3]        []        [8, 12]    []
      (19, 22, 7)                   [9, 16]    [6, 11]    [7, 33]   [32]       [15, 25]
      (1843, 2130, 678)             [22, 27]   [16, 17]   []        [44, 48]   [36, 39]
      (51892, 59973, 19090)         []         []         []        [56]       []
      (113018, 130618, 41577)       []         []         []        []         []
      (114861, 132748, 42255)       [33, 35]   [22, 24]   []        [62, 66]   [51, 53]
      (166753, 192721, 61345)       []         []         []        []         []
      (446524, 516060, 164267)      [36]       [25]       []        []         [56, 57]
      (1174662, 1357589, 432134)    [39, 44]   [26, 29]   []        [68]       [61, 66]
      (3970510, 4588827, 1460669)   []         [28]       []        []         []

The indices in the table are the i-th matrices. For example, the first 3
Dirichlet approximations appear in the convergents of ARP algorithm, but not the
4th neither the 5th)::

    sage: algo = ARP()
    sage: algo.n_matrix((e,pi,1), 3)
    [3 3 2]
    [3 4 2]
    [1 1 1]
    sage: algo.n_matrix((e,pi,1), 6)
    [33 19  8]
    [38 22  9]
    [12  7  3]
    sage: algo.n_matrix((e,pi,1), 16)
    [1631 2498 1843]
    [1885 2887 2130]
    [ 600  919  678]
    sage: algo.n_matrix((e,pi,1), 22)
    [114861 101941  64812]
    [132748 117816  74905]
    [ 42255  37502  23843]
    sage: algo.n_matrix((e,pi,1), 25)
    [446524 331663 842999]
    [516060 383312 974277]
    [164267 122012 310122]
    sage: algo.n_matrix((e,pi,1), 26)
    [1621186  331663 1174662]
    [1873649  383312 1357589]
    [ 596401  122012  432134]
    sage: algo.n_matrix((e,pi,1), 28)
    [3970510 2680987 1174662]
    [4588827 3098490 1357589]
    [1460669  986280  432134]

The Dirichlet vectors can be computed from the precedent ones. So one could
expect a perfect MCF algorithm based on 3x3 matrices::

    sage: from slabbe.diophantine_approx import dirichlet_convergents_dependance
    sage: dirichlet_convergents_dependance([e,pi], 8)
      i   vi                         lin. rec.     remainder
    +---+--------------------------+-------------+-----------+
      0   (3, 3, 1)                  []            (3, 3, 1)
      1   (19, 22, 7)                [6]           (1, 4, 1)
      2   (1843, 2130, 678)          [96, 6]       (1, 0, 0)
      3   (51892, 59973, 19090)      [28, 15, 1]   (0, 0, 0)
      4   (113018, 130618, 41577)    [2, 5, 1]     (0, 0, 0)
      5   (114861, 132748, 42255)    [1, 0, 1]     (0, 0, 0)
      6   (166753, 192721, 61345)    [1, 0, 1]     (0, 0, 0)
      7   (446524, 516060, 164267)   [2, 0, 1]     (0, 0, 0)

But, looking further, it is not true anymore, because v_10 is not a greedy
linear combination of the previous three::

    sage: dirichlet_convergents_dependance([e,pi], 18)   # long time (7.4s)
      i    vi                                  lin. rec.         remainder
    +----+-----------------------------------+-----------------+-----------+
      0    (3, 3, 1)                           []                (3, 3, 1)
      1    (19, 22, 7)                         [6]               (1, 4, 1)
      2    (1843, 2130, 678)                   [96, 6]           (1, 0, 0)
      3    (51892, 59973, 19090)               [28, 15, 1]       (0, 0, 0)
      4    (113018, 130618, 41577)             [2, 5, 1]         (0, 0, 0)
      5    (114861, 132748, 42255)             [1, 0, 1]         (0, 0, 0)
      6    (166753, 192721, 61345)             [1, 0, 1]         (0, 0, 0)
      7    (446524, 516060, 164267)            [2, 0, 1]         (0, 0, 0)
      8    (1174662, 1357589, 432134)          [2, 1, 1]         (0, 0, 0)
      9    (3970510, 4588827, 1460669)         [3, 1]            (0, 0, 0)
      10   (21640489, 25010505, 7961091)       [5, 1, 1, 1]      (0, 0, 0)
      11   (25610999, 29599332, 9421760)       [1, 1]            (0, 0, 0)
      12   (47251488, 54609837, 17382851)      [1, 1]            (0, 0, 0)
      13   (117318127, 135587768, 43158927)    [2, 0, 1, 0, 1]   (0, 0, 0)
      14   (142929126, 165187100, 52580687)    [1, 0, 1]         (0, 0, 0)
      15   (164569615, 190197605, 60541778)    [1, 0, 0, 0, 1]   (0, 0, 0)
      16   (307498741, 355384705, 113122465)   [1, 1]            (0, 0, 0)
      17   (779567097, 900967015, 286786708)   [2, 1]            (0, 0, 0)

Remark: matrices of independant lines is 1 (with Thomas Garrity, 27oct 2016).
He kind of have a proof of that...

This used to give a Value Error::

    sage: dirichlet_convergents_dependance([e,pi], 19) # not tested (1min 12s)
      i    vi                                     lin. rec.             remainder
    +----+--------------------------------------+---------------------+-----------+
      0    (3, 3, 1)                              []                    (3, 3, 1)
      1    (19, 22, 7)                            [6]                   (1, 4, 1)
      2    (1843, 2130, 678)                      [96, 6]               (1, 0, 0)
      3    (51892, 59973, 19090)                  [28, 15, 1]           (0, 0, 0)
      4    (113018, 130618, 41577)                [2, 5, 1]             (0, 0, 0)
      5    (114861, 132748, 42255)                [1, 0, 1]             (0, 0, 0)
      6    (166753, 192721, 61345)                [1, 0, 1]             (0, 0, 0)
      7    (446524, 516060, 164267)               [2, 0, 1]             (0, 0, 0)
      8    (1174662, 1357589, 432134)             [2, 1, 1]             (0, 0, 0)
      9    (3970510, 4588827, 1460669)            [3, 1]                (0, 0, 0)
      10   (21640489, 25010505, 7961091)          [5, 1, 1, 1]          (0, 0, 0)
      11   (25610999, 29599332, 9421760)          [1, 1]                (0, 0, 0)
      12   (47251488, 54609837, 17382851)         [1, 1]                (0, 0, 0)
      13   (117318127, 135587768, 43158927)       [2, 0, 1, 0, 1]       (0, 0, 0)
      14   (142929126, 165187100, 52580687)       [1, 0, 1]             (0, 0, 0)
      15   (164569615, 190197605, 60541778)       [1, 0, 0, 0, 1]       (0, 0, 0)
      16   (307498741, 355384705, 113122465)      [1, 1]                (0, 0, 0)
      17   (779567097, 900967015, 286786708)      [2, 1]                (0, 0, 0)
      18   (8457919940, 9775049397, 3111494861)   [10, 2, 0, 0, 0, 1]   (0, 0, 0)

BENCHMARKS::

    sage: it = best_simultaneous_convergents([e,pi])
    sage: %time L = [next(it) for _ in range(15)]   # not tested (4.66s)
    sage: it = best_simultaneous_convergents([e,pi])
    sage: %time L = [next(it) for _ in range(18)]   # not tested (52s)

AUTHORS:

- Sébastien Labbé, September 22, 2016
- Sébastien Labbé, October 10, 2016
- Sébastien Labbé, October 19, 2016, Cython improvements (10000x faster)
- Sébastien Labbé, October 21, 2016, Parallelization of computations

TODO:

- In general, how many of the dirichlet approximations are found by the MCF algos?
- Code Szekeres MCF algorithm

REFERENCES:

Szekeres, G.
Multidimensional continued fractions.
Ann. Univ. Sci. Budapest. Eötvös Sect. Math. 13 (1970), 113–140 (1971).

Cusick, T. W.
The Szekeres multidimensional continued fraction.
Math. Comp. 31 (1977), no. 137, 280–317.
"""
#*****************************************************************************
#       Copyright (C) 2016 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import itertools
from collections import defaultdict
from bisect import bisect
from sage.functions.other import floor
from sage.misc.functional import round
from sage.modules.free_module_element import vector

def frac(x):
    r"""
    Return the fractional part of real number x.

    Not always perfect...

    EXAMPLES::

        sage: from slabbe.diophantine_approx import frac
        sage: frac(3.2)
        0.200000000000000
        sage: frac(-3.2)
        0.800000000000000
        sage: frac(pi)
        pi - 3
        sage: frac(pi).n()
        0.141592653589793

    This looks suspicious...::

        sage: frac(pi*10**15).n()
        0.000000000000000
    """
    return x - floor(x)

def distance_to_nearest_integer(x):
    raise NotImplementedError

def _best_simultaneous_convergents_upto(v, Q, start=1, verbose=False):
    r"""
    Return the smallest best simultaneous diophantine approximation of vector
    ``v`` at distance ``1/Q``.

    .. WARNING::

        This is a slow function (Python) used for comparison only. Use the
        cython function of the same name in the cython module.

    INPUT:

    - ``v`` -- list of real numbers
    - ``Q`` -- real number, Q>1
    - ``start`` -- integer (default: ``1``), starting value to check
    - ``verbose`` -- boolean (default: ``False``)

    OUTPUT:

    - A tuple (u, R) where u=(p_1, ..., p_n, q) is a vector, q is an
      integer, R is a real number such that coordinates of vector p/q are
      at most 1/R from the coordinates of v.

    EXAMPLES::

        sage: from slabbe.diophantine_approx import _best_simultaneous_convergents_upto
        sage: _best_simultaneous_convergents_upto([e,pi], 2)
        ((3, 3, 1), 3.54964677830384)
        sage: _best_simultaneous_convergents_upto([e,pi], 4)
        ((19, 22, 7), 35.7490143326079)
        sage: _best_simultaneous_convergents_upto([e,pi], 35)
        ((19, 22, 7), 35.7490143326079)
        sage: _best_simultaneous_convergents_upto([e,pi], 36)
        ((1843, 2130, 678), 203.239442934072)
        sage: _best_simultaneous_convergents_upto([e,pi], 203)   # long time (1s)
        ((1843, 2130, 678), 203.239442934072)

    We can start the next computation at step 678::

        sage: _best_simultaneous_convergents_upto([e,pi], 204, start=678) # not tested (25s)
        ((51892, 59973, 19090), 266.167750949912)

    TESTS::

        sage: _best_simultaneous_convergents_upto([1,e,pi], 1)
        Traceback (most recent call last):
        ...
        ValueError: argument Q(=1) must be > 1
    """
    if not Q > 1:
        raise ValueError("argument Q(={}) must be > 1".format(Q))
    v = vector(v)
    d = len(v)
    Qinv = 1. / Q
    un_moins_Qinv = 1 - Qinv
    for q in range(start, Q**d):
        q_v = q*v
        frac_q_v = [frac(a) for a in q_v]
        if verbose:
            print(q,[a.n() for a in frac_q_v])
        if all(a <= Qinv or un_moins_Qinv <= a for a in frac_q_v):
            p = [round(a) for a in q_v]
            p.append(q)
            error = max((a if a < .5 else 1-a) for a in frac_q_v)
            return tuple(p), ~error.n()
    else:
        raise RuntimeError('Did not find diophantine approximation of vector '
                'v={} with parameter Q={}'.format(v, Q))

def best_simultaneous_convergents_upto(v, Q, start=1, verbose=False):
    r"""
    Return a list of all best simultaneous diophantine approximations p,q of vector
    ``v`` at distance ``|qv-p|<=1/Q`` such that `1<=q<Q^d`.

    INPUT:

    - ``v`` -- list of real numbers
    - ``Q`` -- real number, Q>1
    - ``start`` -- integer (default: ``1``), starting value to check
    - ``verbose`` -- boolean (default: ``False``)

    EXAMPLES::

        sage: from slabbe.diophantine_approx import best_simultaneous_convergents_upto
        sage: best_simultaneous_convergents_upto([e,pi], 2)
        [((3, 3, 1), 3.549646778303845)]
        sage: best_simultaneous_convergents_upto([e,pi], 4)
        [((19, 22, 7), 35.74901433260719)]
        sage: best_simultaneous_convergents_upto([e,pi], 36, start=4**2)
        [((1843, 2130, 678), 203.23944293852406)]
        sage: best_simultaneous_convergents_upto([e,pi], 204, start=36**2)
        [((51892, 59973, 19090), 266.16775098010373),
         ((113018, 130618, 41577), 279.18598227531174)]
        sage: best_simultaneous_convergents_upto([e,pi], 280, start=204**2)
        [((114861, 132748, 42255), 412.7859137824949),
         ((166753, 192721, 61345), 749.3634909055199)]
        sage: best_simultaneous_convergents_upto([e,pi], 750, start=280**2)
        [((446524, 516060, 164267), 896.4734658499202),
         ((1174662, 1357589, 432134), 2935.314937919726)]
        sage: best_simultaneous_convergents_upto([e,pi], 2936, start=750**2)
        [((3970510, 4588827, 1460669), 3654.2989332956854),
         ((21640489, 25010505, 7961091), 6257.014011585661)]

    TESTS::

        sage: best_simultaneous_convergents_upto([e,pi], 102300.1, start=10^9) # not tested (1 min)
        [((8457919940, 9775049397, 3111494861), 194686.19839453633)]
    """
    from slabbe.diophantine_approx_pyx import good_simultaneous_convergents_upto
    from sage.parallel.decorate import parallel
    from sage.parallel.ncpus import ncpus
    step = ncpus()
    @parallel
    def F(shift):
        return good_simultaneous_convergents_upto(v, Q, start+shift, step=step)
    shifts = list(range(step))
    goods = []
    for (arg, kwds), output in F(shifts):
        if output == 'NO DATA':
            print ("problem with v={}, Q={}, start={}, shift={}"
                " because output={}".format(v,Q,start,arg[0],output))
        else:
            goods.extend(output)
    if not goods:
        raise ValueError("Did not find an approximation p,q to v={} s.t.  |p-qv|<=1/Q"
            " where Q={}".format(v,Q))
    goods.sort()
    bests = []
    best_error_inv = 0
    # keep only the best ones
    for q in goods:
        q_v = [q*a for a in v]
        frac_q_v = [a-floor(a) for a in q_v]
        error = max((a if a < .5 else 1-a) for a in frac_q_v)
        error_inv = 1. / error.n(digits=50)
        if verbose:
            print("q={}, error_inv={}, best_error_inv={}".format(q,
                error_inv, best_error_inv))
        if error_inv > best_error_inv:
            p = [round(a) for a in q_v]
            p.append(q)
            bests.append((tuple(p),error_inv))
            best_error_inv = error_inv
    return bests

def best_simultaneous_convergents(v):  
    r"""
    Return an iterator of best convergents to a vector of real number according
    to Dirichlet theorem on simultaneous approximations.

    INPUT:

    - ``v`` -- list of real numbers

    OUTPUT:

    - iterator

    EXAMPLES::

        sage: from slabbe.diophantine_approx import best_simultaneous_convergents
        sage: it = best_simultaneous_convergents([e, pi])
        sage: [next(it) for _ in range(5)]
        [(3, 3, 1),
         (19, 22, 7),
         (1843, 2130, 678),
         (51892, 59973, 19090),
         (113018, 130618, 41577)]

    TESTS:

    Correspondance with continued fraction when d=1::

        sage: it = best_simultaneous_convergents([e])
        sage: [next(it) for _ in range(10)]
        [(3, 1),
         (8, 3),
         (11, 4),
         (19, 7),
         (87, 32),
         (106, 39),
         (193, 71),
         (1264, 465),
         (1457, 536),
         (2721, 1001)]
        sage: continued_fraction(e).convergents()[:11].list()
        [2, 3, 8/3, 11/4, 19/7, 87/32, 106/39, 193/71, 1264/465, 1457/536, 2721/1001]
    """
    Q = 2.
    start = 1
    d = len(v)
    while True:
        bests = best_simultaneous_convergents_upto(v, Q, start)
        start = Q**d
        for u,Q in bests:
            yield u
        Q += 0.001 # make sure we do not get the same again

def dirichlet_convergents_dependance(v, n, verbose=False):
    r"""
    INPUT:

    - ``v`` -- list of real numbers
    - ``n`` -- integer, number of iterations
    - ``verbose`` -- bool (default: ``False``),

    OUTPUT:

    - table of linear combinaisons of dirichlet approximations in terms of
      previous dirichlet approximations

    EXAMPLES::

        sage: from slabbe.diophantine_approx import dirichlet_convergents_dependance
        sage: dirichlet_convergents_dependance([e,pi], 4)
          i   vi                      lin. rec.     remainder
        +---+-----------------------+-------------+-----------+
          0   (3, 3, 1)               []            (3, 3, 1)
          1   (19, 22, 7)             [6]           (1, 4, 1)
          2   (1843, 2130, 678)       [96, 6]       (1, 0, 0)
          3   (51892, 59973, 19090)   [28, 15, 1]   (0, 0, 0)

    The last 3 seems enough::

        sage: dirichlet_convergents_dependance([e,pi], 8)
          i   vi                         lin. rec.     remainder
        +---+--------------------------+-------------+-----------+
          0   (3, 3, 1)                  []            (3, 3, 1)
          1   (19, 22, 7)                [6]           (1, 4, 1)
          2   (1843, 2130, 678)          [96, 6]       (1, 0, 0)
          3   (51892, 59973, 19090)      [28, 15, 1]   (0, 0, 0)
          4   (113018, 130618, 41577)    [2, 5, 1]     (0, 0, 0)
          5   (114861, 132748, 42255)    [1, 0, 1]     (0, 0, 0)
          6   (166753, 192721, 61345)    [1, 0, 1]     (0, 0, 0)
          7   (446524, 516060, 164267)   [2, 0, 1]     (0, 0, 0)

    But not in this case::

        sage: dirichlet_convergents_dependance([pi,sqrt(3)], 12)
          i    vi                      lin. rec.                  remainder
        +----+-----------------------+--------------------------+-----------+
          0    (3, 2, 1)               []                         (3, 2, 1)
          1    (22, 12, 7)             [6]                        (4, 0, 1)
          2    (47, 26, 15)            [2, 1]                     (0, 0, 0)
          3    (69, 38, 22)            [1, 1]                     (0, 0, 0)
          4    (176, 97, 56)           [2, 0, 1, 4]               (4, 1, 1)
          5    (223, 123, 71)          [1, 0, 1]                  (0, 0, 0)
          6    (399, 220, 127)         [1, 1]                     (0, 0, 0)
          7    (1442, 795, 459)        [3, 1, 0, 0, 0, 1]         (0, 0, 0)
          8    (6390, 3523, 2034)      [4, 1, 1]                  (0, 0, 0)
          9    (26603, 14667, 8468)    [4, 0, 2, 1, 0, 0, 0, 1]   (0, 0, 0)
          10   (32993, 18190, 10502)   [1, 1]                     (0, 0, 0)
          11   (40825, 22508, 12995)   [1, 0, 1, 1]               (0, 0, 0)

    The v4 is not a lin. comb. of the previous four::

        sage: dirichlet_convergents_dependance([e,pi,sqrt(3)], 5)
          i   vi                                lin. rec.        remainder
        +---+---------------------------------+----------------+--------------+
          0   (3, 3, 2, 1)                      []               (3, 3, 2, 1)
          1   (19, 22, 12, 7)                   [6]              (1, 4, 0, 1)
          2   (193, 223, 123, 71)               [10, 1]          (0, 0, 1, 0)
          3   (5529, 6390, 3523, 2034)          [28, 6, 3]       (2, 5, 1, 1)
          4   (163067, 188461, 103904, 59989)   [29, 14, 1, 1]   (2, 4, 1, 1)
    """
    L = []
    it = best_simultaneous_convergents(v)
    rows = []
    for i in range(n):
        vi = vector(next(it))
        t = vi
        M = []
        for u in reversed(L):
            m = floor(min(a/b for a,b in zip(t,u)))
            M.append(m)
            t -= m*u
            if t == 0:
                if verbose:
                    c = ','.join("v{}".format(len(L)-j) for j in range(len(M)))
                    print("v{} = {} = <{}>.<{}>".format(i, vi, M, c))
                break
        else:
            if verbose:
                print("v{} = {} = <{}>.<v{}, ..., v0> + {}".format(i, vi,
                    M, i-1, t))
        L.append(vi)
        row = [i, vi, M, t]
        rows.append(row)
    header_row = ['i', 'vi', 'lin. rec.', 'remainder']
    from sage.misc.table import table
    return table(rows=rows, header_row=header_row)

def mult_cont_frac_vs_dirichlet_dict(v, dirichlet, algos):
    r"""
    INPUT:

    - ``v`` -- list of real numbers
    - ``dirichlet`` -- list, first dirichlet approximations
    - ``algos`` -- list, list of mult. cont. frac. algorithms

    OUTPUT:

        dict

    EXAMPLES::

        sage: from slabbe.diophantine_approx import best_simultaneous_convergents
        sage: from slabbe.diophantine_approx import mult_cont_frac_vs_dirichlet_dict
        sage: from slabbe.mult_cont_frac import ARP, Brun
        sage: v = [e, pi]
        sage: it = best_simultaneous_convergents(v)
        sage: dirichlet = [next(it) for _ in range(3)]
        sage: mult_cont_frac_vs_dirichlet_dict([e,pi], dirichlet, [Brun(), ARP()])
        {Arnoux-Rauzy-Poincar\'e 3-dimensional continued fraction algorithm: defaultdict(<class 'list'>, {(3, 3, 1): [3], (19, 22, 7): [6, 7, 8, 9, 10, 11], (1843, 2130, 678): [16, 17]}),
         Brun 3-dimensional continued fraction algorithm: defaultdict(<class 'list'>, {(3, 3, 1): [4, 5], (19, 22, 7): [9, 10, 11, 12, 13, 14, 15, 16], (1843, 2130, 678): [22, 23, 24, 25, 26, 27]})}

     Or from precomputed Dirichlet approximations::

        sage: dirichlet = [(3, 3, 1), (19, 22, 7), (1843, 2130, 678), (51892, 59973, 19090)]
        sage: mult_cont_frac_vs_dirichlet_dict([e,pi], dirichlet, [Brun(), ARP()])
        {Arnoux-Rauzy-Poincar\'e 3-dimensional continued fraction algorithm: defaultdict(<class 'list'>, {(3, 3, 1): [3], (19, 22, 7): [6, 7, 8, 9, 10, 11], (1843, 2130, 678): [16, 17]}),
         Brun 3-dimensional continued fraction algorithm: defaultdict(<class 'list'>, {(3, 3, 1): [4, 5], (19, 22, 7): [9, 10, 11, 12, 13, 14, 15, 16], (1843, 2130, 678): [22, 23, 24, 25, 26, 27]})}

    """
    dirichlet_set = set(tuple(u) for u in dirichlet)
    MAX = max(u[-1] for u in dirichlet)
    vv = list(v) + [1]
    D = {}
    for algo in algos:
        D[algo] = defaultdict(list)
        for i in itertools.count():
            columns = algo.n_matrix(vv, i).columns()
            if min(col[-1] for col in columns) > MAX:
                break
            for col in columns:
                col = tuple(col)
                if col in dirichlet_set:
                    D[algo][col].append(i)
    return D

def mult_cont_frac_vs_dirichlet(v, dirichlet, algos):
    r"""
    Returns the indices i such that dirichlet approximations appears as columns
    of the i-th matrix obtained from mult. dim. cont. frac. algorithms.

    INPUT:

    - ``v`` -- list of real numbers
    - ``dirichlet`` -- list, first dirichlet approximations
    - ``algos`` -- list, list of mult. cont. frac. algorithms

    OUTPUT:

    table

    EXAMPLES::

        sage: from slabbe.diophantine_approx import best_simultaneous_convergents
        sage: from slabbe.diophantine_approx import mult_cont_frac_vs_dirichlet
        sage: from slabbe.mult_cont_frac import Brun,ARP,Reverse,Selmer,Cassaigne
        sage: v = [e, pi]
        sage: it = best_simultaneous_convergents(v)
        sage: dirichlet = [next(it) for _ in range(10)]
        sage: algos = [Brun(), ARP(), Reverse(), Selmer(),Cassaigne()]
        sage: mult_cont_frac_vs_dirichlet(v, dirichlet, algos)
          Dirichlet                     Brun       ARP        Reverse   Selmer     Cassaigne
        +-----------------------------+----------+----------+---------+----------+-----------+
          (3, 3, 1)                     [4, 5]     [3]        []        [8, 12]    []
          (19, 22, 7)                   [9, 16]    [6, 11]    [7, 33]   [32]       [15, 25]
          (1843, 2130, 678)             [22, 27]   [16, 17]   []        [44, 48]   [36, 39]
          (51892, 59973, 19090)         []         []         []        [56]       []
          (113018, 130618, 41577)       []         []         []        []         []
          (114861, 132748, 42255)       [33, 35]   [22, 24]   []        [62, 66]   [51, 53]
          (166753, 192721, 61345)       []         []         []        []         []
          (446524, 516060, 164267)      [36]       [25]       []        []         [56, 57]
          (1174662, 1357589, 432134)    [39, 44]   [26, 29]   []        [68]       [61, 66]
          (3970510, 4588827, 1460669)   []         [28]       []        []         []
    """
    D = mult_cont_frac_vs_dirichlet_dict(v, dirichlet, algos)
    rows = []
    for v in dirichlet:
        v = tuple(v)
        row = [v] 
        for algo in algos:
            indices = D[algo][v]
            if len(indices) > 1:
                row.append([min(indices), max(indices)])
            else:
                row.append(indices)
        rows.append(row)
    header_row = ['Dirichlet'] + [algo.class_name() for algo in algos]
    from sage.misc.table import table
    return table(rows=rows, header_row=header_row)







# -*- coding: utf-8 -*-
r"""
Lyapunov parallel computation for MCF algorithms

"""
#*****************************************************************************
#       Copyright (C) 2014-2015 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function

from sage.misc.prandom import random
from sage.parallel.decorate import parallel
from sage.misc.table import table

def lyapunov_sample(algo, n_orbits, n_iterations=1000, ncpus=2,
        verbose=False):
    r"""
    Return lists of values for theta1, theta2 and 1-theta2/theta1 computed
    on many orbits.

    This is computed in parallel.

    INPUT:

    - ``algo`` -- MCF algorithm
    - ``n_orbits`` -- integer, number of orbits
    - ``n_iterations`` -- integer, length of each orbit
    - ``ncpus`` -- integer (default:``2``), number of cpus to use
    - ``verbose`` -- bool (default: ``False``)

    OUTPUT:

        tuple of three lists

    EXAMPLES::

        sage: from slabbe.lyapunov import lyapunov_sample
        sage: from slabbe.mult_cont_frac import Brun
        sage: lyapunov_sample(Brun(), 5, 1000000) # abs tol 0.01
        [(0.3027620661266397,
          0.3033468535021702,
          0.3044950176856005,
          0.3030531162480779,
          0.30601169862996064),
         (-0.11116236859835525,
          -0.11165563059874498,
          -0.1122595926203868,
          -0.11190323336181864,
          -0.11255687513610782),
         (1.367160820443926,
          1.3680790794750939,
          1.3686746452327765,
          1.3692528714016428,
          1.3678188632657973)]

    Works for higher dimensional algorithms::

        sage: lyapunov_sample(Brun(8), 5, 10^5) # abs tol 0.01
        [(0.24494574466175367,
          0.24492293068699247,
          0.24494468166245503,
          0.2447894172680422,
          0.2452400265773239),
         (-0.012028663893597698,
          -0.012174084683987084,
          -0.012268030566904951,
          -0.012286468841900471,
          -0.012166894088285442),
         (1.049107462185996,
          1.049705777445336,
          1.0500849027773989,
          1.0501919935061854,
          1.0496121871217023)]

    """
    dim = algo.dimension()
    S = [tuple(random() for _ in range(dim)) for _ in range(n_orbits)]
    @parallel(ncpus=ncpus)
    def compute_exponents(i):
        try:
            # When start=None, a random starting point is chosen
            # but it is always the same(!) because the random seed is
            # reinitialize to the same value each time in @parallel. 
            # This is why we need the list S of random starting points
            return algo.lyapunov_exponents(start=S[i], n_iterations=n_iterations) 
        except Exception as err:
            return "{}: {}".format(err.__class__.__name__, err)
    inputs = list(range(n_orbits))
    L = [v for _,v in compute_exponents(inputs)]
    L_filtered = [v for v in L if isinstance(v, tuple)]
    if verbose:
        L_error_msg = [v for v in L if not isinstance(v, tuple)]
        print(L_error_msg)
    return list(zip(*L_filtered))

def lyapunov_table(algo, n_orbits, n_iterations=1000, ncpus=2):
    r"""
    Return a table of values of Lyapunov exponents for this algorithm.

    INPUT:

    - ``algo`` -- MCF algorithm
    - ``n_orbits`` -- integer, number of orbits
    - ``n_iterations`` -- integer, length of each orbit
    - ``ncpus`` -- integer (default:``2``), number of cpus to use

    OUTPUT:

        table of liapounov exponents

    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Brun
        sage: from slabbe.lyapunov import lyapunov_table
        sage: lyapunov_table(Brun(), 10, 1000000) # random
          10 succesful orbits     min       mean      max       std
        +-----------------------+---------+---------+---------+---------+
          $\theta_1$              0.303     0.305     0.307     0.0013
          $\theta_2$              -0.1131   -0.1124   -0.1115   0.00051
          $1-\theta_2/\theta_1$   1.3678    1.3687    1.3691    0.00043

    Works for higher dimensional algorithms::

        sage: lyapunov_table(Brun(8), 10, 10^6) # random
          10 succesful orbits     min        mean       max        std
        +-----------------------+----------+----------+----------+----------+
          $\theta_1$              0.24491    0.24500    0.24506    0.000041
          $\theta_2$              -0.01230   -0.01211   -0.01198   0.000096
          $1-\theta_2/\theta_1$   1.0489     1.0494     1.0502     0.00040

    """
    import numpy as np
    from sage.misc.functional import numerical_approx
    from sage.rings.real_mpfr import RR
    from sage.misc.table import table
    rep = lyapunov_sample(algo, n_orbits, n_iterations, ncpus=ncpus)
    def floor_log(number):
        return RR(number).abs().log10().floor()
    def my_rounded(number, s):
        m = floor_log(number)
        return numerical_approx(number, digits=m-s+1)
    names = [r"$\theta_1$", r"$\theta_2$", r"$1-\theta_2/\theta_1$"]
    rows = []
    for i, data in enumerate(rep):
        data = np.array(data)
        s = floor_log(data.std())
        row = [names[i]]
        row.append(my_rounded(data.min(),s))
        row.append(my_rounded(data.mean(),s))
        row.append(my_rounded(data.max(),s))
        row.append(my_rounded(data.std(),s))
        rows.append(row)
    header = ['{} succesful orbits'.format(len(rep[0])), 'min','mean','max','std']
    return table(rows=rows,header_row=header)

def _lyapunov_row(algo, n_orbits, n_iterations=1000, ncpus=2):
    r"""
    Return a row of values of Lyapunov exponents.

    This method is used for making a larger table comparing many
    algorithms.

    INPUT:

    - ``algo`` -- MCF algorithm
    - ``n_orbits`` -- integer, number of orbits
    - ``n_iterations`` -- integer, length of each orbit
    - ``ncpus`` -- integer (default:``2``), number of cpus to use

    OUTPUT:

        row of liapounov exponents: theta1, theta2, 1-theta2/theta1

    EXAMPLES::

        sage: from slabbe.lyapunov import _lyapunov_row
        sage: from slabbe.mult_cont_frac import Brun
        sage: _lyapunov_row(Brun(), 10, 10^6) # abs tol 0.01
        ['Brun (d=3)', 10, '0.305 (0.0013)', '-0.1121 (0.00061)', '1.368 (0.0010)']

    Works for higher dimensional algorithms::

        sage: _lyapunov_row(Brun(8), 10, 10^6) # abs tol 0.01
        ['Brun (d=8)',
         10,
         '0.24502 (0.000058)',
         '-0.01204 (0.000069)',
         '1.0491 (0.00027)']

    """
    import numpy as np
    from sage.misc.functional import numerical_approx
    from sage.rings.real_mpfr import RR
    rep = lyapunov_sample(algo, n_orbits, n_iterations, ncpus=ncpus)
    def floor_log(number):
        return RR(number).abs().log10().floor()
    def my_rounded(number, s):
        m = floor_log(number)
        return numerical_approx(number, digits=m-s+1)
    row = []
    row.append("{} (d={})".format(algo.name(),algo.dimension()))
    row.append(len(rep[0]))
    for data in rep:
        data = np.array(data)
        s = floor_log(data.std())
        val = my_rounded(data.mean(), s)
        std = my_rounded(data.std(), s)
        row.append("{} ({})".format(val, std))
    return row

def lyapunov_comparison_table(L, n_orbits=100, n_iterations=10000, ncpus=2):
    r"""
    Return a table of values of Lyapunov exponents for many algorithm.

    INPUT:

    - ``L`` -- list of algorithms
    - ``n_orbits`` -- integer
    - ``n_iterations`` -- integer
    - ``ncpus`` -- integer (default:``2``), number of cpus to use

    OUTPUT:

        table

    EXAMPLES::

        sage: import slabbe.mult_cont_frac as mcf
        sage: from slabbe.lyapunov import lyapunov_comparison_table
        sage: algos = [mcf.Brun(), mcf.ARP()]
        sage: lyapunov_comparison_table(algos)    # abs tol 0.01
          Algorithm                       \#Orbits   $\theta_1$ (std)   $\theta_2$ (std)   $1-\theta_2/\theta_1$ (std)
        +-------------------------------+----------+------------------+------------------+-----------------------------+
          Arnoux-Rauzy-Poincar\'e (d=3)   100        0.44 (0.014)       -0.173 (0.0060)    1.389 (0.0051)
          Brun (d=3)                      100        0.305 (0.0085)     -0.112 (0.0042)    1.368 (0.0073)


    Works for higher dimensional algorithms::

        sage: algos = [mcf.Brun(a) for a in range(3,6)]
        sage: lyapunov_comparison_table(algos)    # abs tol 0.01
          Algorithm    \#Orbits   $\theta_1$ (std)   $\theta_2$ (std)   $1-\theta_2/\theta_1$ (std)
        +------------+----------+------------------+------------------+-----------------------------+
          Brun (d=3)   100        0.304 (0.0083)     -0.112 (0.0035)    1.369 (0.0068)
          Brun (d=4)   100        0.326 (0.0023)     -0.072 (0.0018)    1.221 (0.0049)
          Brun (d=5)   100        0.309 (0.0010)     -0.046 (0.0012)    1.150 (0.0037)

    """
    rows = []
    for algo in L:
        try:
            row = _lyapunov_row(algo, n_orbits, n_iterations, ncpus=ncpus)
        except Exception as err:
            s = "{}: {}".format(err.__class__.__name__, err)
            print("Ignoring {} in Lyapunov table. {}".format(algo.class_name(), s))
        else:
            rows.append(row)
    rows.sort(key=lambda d:d[4], reverse=True)
    header = ("Algorithm", r"\#Orbits", r"$\theta_1$ (std)", r"$\theta_2$ (std)",
               r"$1-\theta_2/\theta_1$ (std)")
    return table(rows=rows, header_row=header)




# -*- coding: utf-8 -*-
r"""
Writing to files

EXAMPLES::

    sage: from slabbe.write_to_file import write_str_to_file
    sage: filename = tmp_filename(ext='.tex')
    sage: write_str_to_file('aaa', filename)
    Creation of ...tex

AUTHOR:

    - Sébastien Labbé, March 13, 2018
"""
#*****************************************************************************
#       Copyright (C) 2018 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function

def write_str_to_file(s, filename):
    r"""
    INPUT:

    - ``s`` -- string
    - ``filename`` -- string, filename

    EXAMPLES::

        sage: from slabbe.write_to_file import write_str_to_file
        sage: filename = tmp_filename(ext='.tex')
        sage: write_str_to_file('aaa', filename)
        Creation of ...tex
    """
    with open(filename, 'w') as f:
        print("Creation of {}".format(filename))
        f.write(s)

def print_set_to_file(s, filename, ncolumns=4, start=r'\{', end=r'\}',
        sep=r',', eol=r',\\&\hspace{10mm}'):
    r"""
    INPUT:

    - ``s`` -- string
    - ``filename`` -- string, filename

    EXAMPLES::

        sage: from slabbe.write_to_file import print_set_to_file
        sage: filename = tmp_filename(ext='.tex')
        sage: M = [random_matrix(ZZ, 3, 3) for _ in range(20)]
        sage: print_set_to_file(M, filename, ncolumns=8, 
        ....:          start=r'\left\{\begin{array}{cccccccc}',
        ....:          end=r'\end{array}\right\}',
        ....:          sep=r',&', eol=r',\\[4mm]')
        Creation of ...tex

    """
    from sage.misc.latex import latex
    L = [start]
    for i,a in enumerate(s):
        L.append(latex(a))
        if (i+1) == len(s):
            L.append(end)
        elif (i+1) % ncolumns == 0:
            L.append(eol)
        else:
            L.append(sep)
    output = '\n'.join(L)
    write_str_to_file(output, filename)




# -*- coding: utf-8 -*-
r"""
Finite words

Methods that are not in Sage (for now!)

AUTHORS:

- Sébastien Labbé, 2015
- Sébastien Labbé, 2017, added lexicographic Lyndon stuff

EXAMPLES::

    sage: from slabbe.finite_word import discrepancy
    sage: w = words.ChristoffelWord(5,8)
    sage: discrepancy(w)
    12/13
"""
#*****************************************************************************
#       Copyright (C) 2015,2017 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import itertools
from collections import Counter
from random import randrange
from sage.rings.rational_field import QQ
from sage.functions.other import abs

def discrepancy(self, freq=None):
    r"""
    Return the discrepancy of the word.
    
    This is a distance to the euclidean line defined in [T1980]_.

    INPUT:

    - ``freq`` -- frequency vector (default: ``None``)

    EXAMPLES::

        sage: from slabbe.finite_word import discrepancy
        sage: w = words.ChristoffelWord(5,8)
        sage: w
        word: 0010010100101
        sage: discrepancy(w)
        12/13

    ::

        sage: for c in w.conjugates(): print (c, discrepancy(c))
        0010010100101 12/13
        0100101001010 7/13
        1001010010100 10/13
        0010100101001 10/13
        0101001010010 7/13
        1010010100100 12/13
        0100101001001 8/13
        1001010010010 9/13
        0010100100101 11/13
        0101001001010 6/13
        1010010010100 11/13
        0100100101001 9/13
        1001001010010 8/13

    REFERENCES:

    .. [T1980] R., Tijdeman. The chairman assignment problem. Discrete
       Mathematics 32, no 3 (1980): 323-30. doi:10.1016/0012-365X(80)90269-1.
    """
    length = self.length()
    if freq is None:
        freq = {a:QQ((v,length)) 
                for (a,v) in self.evaluation_dict().items()}
    C = Counter()
    M = 0
    for i,a in enumerate(self):
        C[a] += 1
        MM = max(abs(freq[b]*(i+1) - C[b]) for b in freq)
        M = max(M, MM)
    return M

def to_image(self, width=1000):
    r"""
    Creates an image from a word

    INPUT:

    - ``width`` -- integer, width of image

    EXAMPLES::

        sage: from slabbe.finite_word import to_image
        sage: t = words.ThueMorseWord()
        sage: img = to_image(t[:10000], width=100)
        sage: img
        <PIL.Image.Image image mode=RGB size=100x100 at 0x...>
        sage: img.show()    # not tested

    ::

        sage: W = FiniteWords(range(10))
        sage: d = {a:W.random_element(7) for a in range(10)}
        sage: m = WordMorphism(d, codomain=W)
        sage: w = m.periodic_points()[0][0]

    ::

        sage: s = str(pi.n(digits=40001))[2:]
        sage: s = [int(a) for a in s]
        sage: len(s)
        40000
        sage: img = to_image(W(s), 200)
        sage: img.show()    # not tested
    """
    #http://stackoverflow.com/questions/434583/what-is-the-fastest-way-to-draw-an-image-from-discrete-pixel-values-in-python
    import numpy as np

    height = self.length() // width

    # Create a 1024x1024x3 array of 8 bit unsigned integers
    data = np.zeros( (height,width,3), dtype=np.uint8 )
    data += 255   # white as default color
    alphabet = self.parent().alphabet()

    color_dict={a:[randrange(256) for _ in range(3)] for a in alphabet}

    for i,a in enumerate(self):
        x = i % width
        y = i // width
        data[y,x] = color_dict[a]

    from PIL import Image
    img = Image.fromarray(data)
    #img.show()                     # View in default viewer
    return img

def run_length_encoding(self):
    r"""
    EXAMPLES::

        sage: from slabbe.finite_word import run_length_encoding
        sage: L = [0, 1, 1, 1, 1, 1, 1, 2, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3]
        sage: run_length_encoding(L)
        [(0, 1), (1, 6), (2, 1), (1, 1), (2, 5), (3, 6)]
    """
    return [(value, sum(1 for _ in it)) for value,it in itertools.groupby(self)]

def word_to_polyomino(self):
    r"""
    Returns the inside points of a polyomino.

    INPUT:

    - ``self`` -- list of integers in 0,1,2,3 describing a closed path

    OUTPUT:

    - list of 2d vectors

    EXAMPLES::

        sage: from slabbe.finite_word import word_to_polyomino
        sage: w = [0,0,0,1,1,1,2,2,2,3,3,3]
        sage: sorted(word_to_polyomino(w))
        [(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (1, 2), (2, 0), (2, 1), (2, 2)]
        sage: w = [1,1,1,0,0,0,3,3,3,2,2,2]
        sage: sorted(word_to_polyomino(w))
        [(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (1, 2), (2, 0), (2, 1), (2, 2)]
    """
    from sage.combinat.words.words import FiniteWords
    alphabet = [0,1,2,3]
    FW = FiniteWords(alphabet)
    w = FW(self)

    # Checking it is closed
    end = (w.count(0)-w.count(2), w.count(1) - w.count(3))
    if end != (0,0):
        raise ValueError("input word is not closed path, "
                "it finishes at {}".format(end))

    # Checking its turning number is 1 = 4/4
    w_ = w + FW([self[0]])
    turns = w_.finite_differences(mod=4)
    turning_number = turns.count(1) - turns.count(3)
    if turning_number == 4:
        pass
    elif turning_number == -4:
        self = [(a+2)%4 for a in reversed(self)]
    else:
        raise ValueError("input word is not simple path, "
                "its turning number is {}".format(turning_number))

    # Setting up directions, outside and inside points of each edge
    from sage.modules.free_module import FreeModule
    from sage.rings.integer_ring import ZZ
    F = FreeModule(ZZ, 2)
    directions = list(map(F, [(1,0), (0,1), (-1,0), (0,-1)]))
    outside = list(map(F, [(0,-1), (0,0), (-1,0), (-1,-1)]))
    inside = list(map(F, [(0,0), (-1,0), (-1,-1), (0,-1)]))
    # obtained from below after a translation by (-1/2,-1/2)
    #outside = map(F, [(u/2,-u/2), (u/2,u/2), (-u/2,u/2), (-u/2,-u/2)])
    #inside = map(F, [(u/2,u/2), (-u/2,u/2), (-u/2,-u/2), (u/2,-u/2)])
    for p in directions: p.set_immutable()
    for p in outside: p.set_immutable()
    for p in inside: p.set_immutable()
    alph_to_dir = dict(zip(alphabet, directions))
    alph_to_outside = dict(zip(alphabet, outside))
    alph_to_inside = dict(zip(alphabet, inside))

    # Computing the outside points
    cur = F.zero()
    outside_points = set()
    for a in self:
        outside = cur + alph_to_outside[a]
        outside.set_immutable()
        outside_points.add(outside)
        cur += alph_to_dir[a]

    # The recursively enumerated set
    def children(p):
        possible = []
        for d in directions:
            p_d = p+d
            p_d.set_immutable()
            if p_d not in outside_points:
                possible.append(p_d)
        return possible
    seeds = [alph_to_inside[self[0]]]
    from sage.sets.recursively_enumerated_set import RecursivelyEnumeratedSet
    R = RecursivelyEnumeratedSet(seeds, children, structure='symmetric')
    return list(R)

def minimum_lexicographic_conjugate_reversal(self):
    r"""
    TODO: Use Lyndon factorisation to improve the time/space...

    EXAMPLES::

        sage: from slabbe.finite_word import minimum_lexicographic_conjugate_reversal
        sage: minimum_lexicographic_conjugate_reversal(Word([1,3,2,2,2]))
        word: 12223

    ::

        sage: w = words.ChristoffelWord(72452,462443)
        sage: minimum_lexicographic_conjugate_reversal(w)
        word: 0000000100000010000000100000010000001000...
        sage: _ == w
        True
        sage: minimum_lexicographic_conjugate_reversal(Word([1,3,2,2,1,1,2]))
        word: 1121322
    """
    u = minimum_lexicographic_conjugate(self)
    v = minimum_lexicographic_conjugate(self.reversal())
    return min((u,v))

def minimum_lexicographic_conjugate(self):
    r"""
    Return the conjugate word which is minimal for the lexicographic order.

    The output is a Lyndon word (or some power of).

    EXAMPLES::

        sage: from slabbe.finite_word import minimum_lexicographic_conjugate
        sage: minimum_lexicographic_conjugate(Word([1,3,2,2,2]))
        word: 13222
        sage: minimum_lexicographic_conjugate(Word([1,4,3]))
        word: 143
        sage: minimum_lexicographic_conjugate(Word([3,4,1]))
        word: 134

    The code is fast::

        sage: w = words.ChristoffelWord(72452, 462443)
        sage: minimum_lexicographic_conjugate(w)
        word: 0000000100000010000000100000010000001000...
    """
    from sage.misc.misc_c import prod
    from sage.combinat.words.word import Word
    f = self.lyndon_factorization()
    return f[-1]*prod(f[:-1], Word())

def is_lyndon_mod_reverse(self):
    r"""
    EXAMPLES::

        sage: from slabbe.finite_word import is_lyndon_mod_reverse
        sage: is_lyndon_mod_reverse(Word('111222'))
        True
        sage: is_lyndon_mod_reverse(Word('1112221'))
        False
        sage: is_lyndon_mod_reverse(Word('143'))
        False

    ::

        sage: w = words.ChristoffelWord(72452,462443)
        sage: is_lyndon_mod_reverse(w)
        True
    """
    return self.is_lyndon() and self <= minimum_lexicographic_conjugate(self.reversal())

def are_overlapping_factors(u, v, d):
    r"""
    Returns whether there exists a word w such that u occurs in w at position 0
    and v occurs in w at position d.

    INPUT:

    - ``u`` -- word
    - ``v`` -- word
    - ``d`` -- integer (positive or negative)

    EXAMPLES::

        sage: from slabbe.finite_word import are_overlapping_factors
        sage: are_overlapping_factors('abc', 'abc', 0)
        True
        sage: are_overlapping_factors('abc', 'abc', 1)
        False
        sage: are_overlapping_factors('abcdef', 'bc', 1)
        True
        sage: are_overlapping_factors('abcdef', 'bcdefgh', 1)
        True
        sage: are_overlapping_factors('abcdef', 'bcddefgh', 1)
        False
        sage: are_overlapping_factors('abcdef', 'aabcdefgh', -1)
        True
        sage: are_overlapping_factors('abcdef', 'aabcd', -1)
        True
        sage: are_overlapping_factors('abcdef', 'aabcc', -1)
        False

    """
    start_u = max(0,d)
    end_u = min(len(u),d+len(v))
    fact_u = u[start_u:end_u]
    fact_v = v[start_u-d:end_u-d]
    return fact_u == fact_v

def sort_word_by_length_lex_key(w):
    r"""
    A key function to sort word (by length, and then lexicographically).

    EXAMPLES::

        sage: from slabbe.finite_word import sort_word_by_length_lex_key
        sage: L = ['aa', 'aaa', 'bb', 'ccc']
        sage: sorted(L, key=sort_word_by_length_lex_key)
        ['aa', 'bb', 'aaa', 'ccc']
    """
    return (len(w), w)



# -*- coding: utf-8 -*-
r"""
Functions on graphs

.. TODO::

    - Make the doctests more simple
"""
#*****************************************************************************
#       Copyright (C) 2016-2017 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from collections import Counter, defaultdict
import itertools
from sage.graphs.digraph import DiGraph
from sage.graphs.graph import Graph

def projection_graph(G, proj_fn, filename=None, verbose=False):
    r"""
    Return the image of a graph under a function on vertices.

    INPUT:

    - ``G`` -- graph
    - ``proj_fn`` -- function
    - ``filename`` -- integer (default:``None``), save the graph to this pdf
      filename if filename is not None
    - ``verbose`` -- bool (default:``False``), print a table of data about the
      projection

    EXAMPLES::

        sage: from slabbe.graph import projection_graph
        sage: g = graphs.PetersenGraph()
        sage: g.vertices()
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        sage: f = lambda i: i % 5
        sage: projection_graph(g, f)
        Looped multi-digraph on 5 vertices

    With verbose information::

        sage: projection_graph(g, lambda i:i%4, verbose=True)
          Number of vertices   Projected vertices
        +--------------------+--------------------+
          2                    3
          2                    2
          3                    1
          3                    0
        Looped multi-digraph on 4 vertices
    """
    edges = set((proj_fn(A),proj_fn(B)) for A,B,_ in G.edges())
    G_proj = DiGraph(edges, format='list_of_edges', loops=True, multiedges=True)
    if verbose:
        d = dict(Counter(proj_fn(s) for s in G.vertices()))
        rows = [(value, key) for key,value in d.items()]
        rows.sort(reverse=True,key=lambda row:row[1])
        header_row = ['Number of vertices', 'Projected vertices']
        from sage.misc.table import table
        print(table(rows=rows, header_row=header_row))
    if filename:
        from slabbe import TikzPicture
        print(TikzPicture.from_graph(G_proj, prog='dot').pdf(filename))
    return G_proj

def digraph_move_label_to_edge(G, label_function=None, loops=True,
        multiedges=False):
    r"""
    Return a digraph with labels moved from the arrival vertices to
    corresponding edges.

    INPUT:

    - ``G`` -- graph, whose vertices are tuples of the form (vertex, label)
    - ``label_function`` -- function or None, a function to apply to each label
    - ``loops`` -- bool (default: True)
    - ``multiedges`` -- bool (default: False)

    EXAMPLES::

        sage: G = DiGraph()
        sage: G.add_edges([((i, None), ((i+1)%10, 'plusone')) for i in range(10)])
        sage: G.add_edges([((i, None), ((i+2)%10, 'plustwo')) for i in range(10)])
        sage: G
        Digraph on 30 vertices
        sage: from slabbe.graph import digraph_move_label_to_edge
        sage: digraph_move_label_to_edge(G)
        Looped digraph on 10 vertices

    Using a function to modify the labels::

        sage: f = lambda label:"A"+label
        sage: GG = digraph_move_label_to_edge(G, label_function=f)
        sage: GG
        Looped digraph on 10 vertices
        sage: GG.edges()[0]
        (0, 1, 'Aplusone')
    """
    if label_function:
        edges = [(u,v,label_function(label)) for ((u,_), (v,label), _) in G.edges()]
    else:
        edges = [(u,v,label) for ((u,_), (v,label), _) in G.edges()]
    return DiGraph(edges, format='list_of_edges', loops=loops,
            multiedges=multiedges)

def induced_subgraph(G, filter):
    r"""
    Return the induced subdigraph of a digraph keeping only vertices that are
    map to ``True`` by the filter.

    INPUT:

    - ``G`` -- graph
    - ``filter`` -- function, a function from vertices to boolean

    EXAMPLES::

        sage: from slabbe.graph import induced_subgraph
        sage: G = DiGraph()
        sage: G.add_edges([((i, ''), ((i+1)%10, 'plusone')) for i in range(10)])
        sage: G.add_edges([((i, ''), ((i+2)%10, 'plustwo')) for i in range(10)])
        sage: GG = induced_subgraph(G, lambda v: v[0]%2 == 0)
        sage: G
        Digraph on 30 vertices
        sage: GG
        Digraph on 15 vertices
        sage: GG.edges()[0]
        ((0, ''), (2, 'plustwo'), None)

    .. TODO::

        simplify the edges aaaaa*aaa*aaa* to a* only
    """
    GG = G.copy()
    loops = dict((u, label) for (u,v,label) in GG.loop_edges())
    for v in GG.vertices():
        if filter(v):
            continue
        incoming = [(x,y,l) for (x,y,l) in GG.incoming_edges(v) if x != y]
        outgoing = [(x,y,l) for (x,y,l) in GG.outgoing_edges(v) if x != y]
        GG.delete_vertex(v)
        if v in loops:
            it = itertools.product(outgoing, [(loops[v][0]*10,)], incoming) 
        else:
            it = itertools.product(outgoing, [tuple()], incoming) 
        for c,b,a in it:
            _,y,labelout = c
            x,_,labelin = a
            GG.add_edge(x,y, labelout + b + labelin)
    return GG

def merge_multiedges(G, label_function=tuple):
    r"""
    Return the (di)graph where multiedges are merged into one.

    INPUT:

    - ``G`` -- graph
    - ``label_function`` -- function (default:``tuple``), a function to
      apply to each list of labels

    OUTPUT:

        (looped) (di)graph

    EXAMPLES:

    A digraph::

        sage: from slabbe.graph import merge_multiedges
        sage: G = DiGraph(multiedges=True)
        sage: G.add_edge(0,1,'one')
        sage: G.add_edge(0,1,'two')
        sage: G.add_edge(0,1,'alpha')
        sage: GG = merge_multiedges(G)
        sage: GG
        Digraph on 2 vertices
        sage: GG.edges()
        [(0, 1, ('alpha', 'one', 'two'))]

    A graph::

        sage: G = Graph(multiedges=True)
        sage: G.add_edge(0,1,'one')
        sage: G.add_edge(0,1,'two')
        sage: G.add_edge(0,1,'alpha')
        sage: GG = merge_multiedges(G)
        sage: GG
        Graph on 2 vertices
        sage: GG.edges()
        [(0, 1, ('alpha', 'one', 'two'))]

    Using ``label_function``::

        sage: fn = lambda L: LatexExpr(','.join(map(str, L)))
        sage: GG = merge_multiedges(G, label_function=fn)
        sage: GG.edges()
        [(0, 1, alpha,one,two)]

    """
    d = defaultdict(list)
    for (u,v,label) in G.edges():
        d[(u,v)].append(label)

    edges = [(u,v,label_function(label_list)) for (u,v),label_list in d.items()]

    loops = G.has_loops()
    if G.is_directed():
        return DiGraph(edges, format='list_of_edges', loops=loops)
    else:
        return Graph(edges, format='list_of_edges', loops=loops)

def clean_sources_and_sinks(G):
    r"""
    Return a copy of the graph where every vertices of the graph that have
    in or out degree 0 is removed (recursively).

    EXAMPLES::

        sage: from slabbe.graph import clean_sources_and_sinks
        sage: L = [(0,1),(1,2),(2,3),(3,4),(4,5),(5,3)]
        sage: G = DiGraph(L,format='list_of_edges')
        sage: H = clean_sources_and_sinks(G)
        sage: H
        Digraph on 3 vertices
        sage: H.vertices()
        [3, 4, 5]

    ::

        sage: L = [(0,1),(1,2),(2,3),(3,4),(4,5),(5,3),(1,0)]
        sage: G = DiGraph(L, format='list_of_edges')
        sage: H = clean_sources_and_sinks(G)
        sage: H
        Digraph on 6 vertices
        sage: H.vertices()
        [0, 1, 2, 3, 4, 5]

    """
    H = G.copy()
    done = False
    while not done:
        done = True
        for v in H.vertices():
            if H.in_degree(v) == 0 or H.out_degree(v) == 0:
                done = False
                H.delete_vertex(v)
    return H

def get_funnel(G):
    r"""
    Return an edge (u,v) such that u and v are distinct, G.out_degree(u) is
    1 and G.in_degree(v) is 1. Return ``None`` if no such funnel is found.

    INPUT:

    - ``G`` -- digraph

    EXAMPLES::

        sage: from slabbe.graph import get_funnel
        sage: G = DiGraph([(str(a),str(a+1)) for a in range(5)], format='list_of_edges')
        sage: get_funnel(G)
        ('0', '1')
    """
    for (u,v,_) in G.edges(): 
        if (u != v and G.in_degree(v) == 1 and G.out_degree(u) == 1):
            return (u,v)
    else:
        return None

def reduce_funnel_edges(G, merge_function):
    r"""
    Reduce a graph by merging all funnel edge.

    We say that an edge (u,v) is a "funnel" edge if u is not v
    and the out degree of u and the in degree of v are both equal to 1.

    INPUT:

    - ``G`` -- digraph
    - ``merge_function`` -- function taking two vertices as input and
      returning a new vertex

    EXAMPLES::

        sage: from slabbe.graph import reduce_funnel_edges
        sage: G = DiGraph([(str(a),str(a+1)) for a in range(5)], format='list_of_edges')
        sage: merge_function = lambda a,b:a+b
        sage: GG = reduce_funnel_edges(G, merge_function)
        sage: GG.vertices()
        ['012345']

    ::

        sage: G = DiGraph([(str(a),str((a+1)%5)) for a in range(5)], format='list_of_edges')
        sage: merge_function = lambda a,b:a+b
        sage: GG = reduce_funnel_edges(G, merge_function)
        sage: GG.vertices()
        ['01234']

    The following result does not seem right::

        sage: w = words.FibonacciWord()[:100]
        sage: G = w.rauzy_graph(11)
        sage: merge_function = lambda a,b:a+b[-1:]
        sage: GG = reduce_funnel_edges(G, merge_function)
        sage: GG.vertices()
        [word: 01001010010, word: 100101001001, word: 100101001011]

    """
    from copy import copy
    GG = copy(G)
    GG.allow_loops(True)
    #GG.allow_multiple_edges(True)

    funnel = get_funnel(GG)
    while funnel:
        (u,v) = funnel
        u_v = merge_function(u, v)
        GG.add_vertex(u_v)
        for s in GG.neighbors_out(v):
            GG.add_edge(u_v, s)
        for s in GG.neighbors_in(u):
            GG.add_edge(s, u_v)
        GG.delete_vertex(u)
        GG.delete_vertex(v)
        funnel = get_funnel(GG)
    return GG

def get_left_special_vertex(G):
    r"""
    Return a vertex v such that v is left special but not bispecial, that is,
    ``G.in_degree(v)>1`` but ``G.out_degree(v)<=1``.

    Return ``None`` if no such vertex is found.

    INPUT:

    - ``G`` -- digraph

    OUTPUT:

    a vertex or ``None`` if no such vertex is found.

    EXAMPLES::

        sage: from slabbe.graph import get_left_special_vertex
        sage: G = DiGraph([(5,6), (6,7), (6,8)], format='list_of_edges')
        sage: get_left_special_vertex(G) is None
        True
        sage: G = DiGraph([(6,5), (7,6), (8,6)], format='list_of_edges')
        sage: get_left_special_vertex(G)
        6

    If there is a bispecial, but no left special it returns ``None``::

        sage: G = DiGraph([(2,3),(3,4),(4,2),(2,5),(5,6),(6,2)], format='list_of_edges')
        sage: get_left_special_vertex(G) is None
        True

    """
    for v in G.vertices():
        if G.in_degree(v) > 1 and G.out_degree(v) <= 1:
            return v
    else:
        return None

def get_right_special_vertex(G):
    r"""
    Return a vertex v such that v is right special but not bispecial, that is,
    ``G.in_degree(v)<=1`` but ``G.out_degree(v)>1``.

    Return ``None`` if no such vertex is found.

    INPUT:

    - ``G`` -- digraph

    OUTPUT:

    a vertex or ``None`` if no such vertex is found.

    EXAMPLES::

        sage: from slabbe.graph import get_right_special_vertex
        sage: G = DiGraph([(5,6), (6,7), (6,8)], format='list_of_edges')
        sage: get_right_special_vertex(G)
        6
        sage: G = DiGraph([(6,5), (7,6), (8,6)], format='list_of_edges')
        sage: get_right_special_vertex(G) is None
        True

    """
    for v in G.vertices():
        if G.in_degree(v) <= 1 and G.out_degree(v) > 1:
            return v
    else:
        return None

def get_bispecial_vertex(G):
    r"""
    Return a vertex v such that v is bispecial, that is,
    ``G.in_degree(v)>1`` but ``G.out_degree(v)>1``.

    Return ``None`` if no such vertex is found.

    INPUT:

    - ``G`` -- digraph

    OUTPUT:

    a vertex or ``None`` if no such vertex is found.

    EXAMPLES::

        sage: from slabbe.graph import get_bispecial_vertex
        sage: G = DiGraph([(4,6), (5,6), (6,7), (6,8)], format='list_of_edges')
        sage: get_bispecial_vertex(G)
        6
        sage: G = DiGraph([(6,5), (7,6), (8,6)], format='list_of_edges')
        sage: get_bispecial_vertex(G) is None
        True

    """
    for v in G.vertices():
        if G.in_degree(v) > 1 and G.out_degree(v) > 1:
            return v
    else:
        return None

def bispecial_vertices(G):
    r"""
    Return the list of vertices v such that v is bispecial, that is,
    ``G.in_degree(v)>1`` but ``G.out_degree(v)>1``.

    INPUT:

    - ``G`` -- digraph

    EXAMPLES::

        sage: from slabbe.graph import bispecial_vertices
        sage: G = DiGraph([(4,6), (5,6), (6,7), (6,8)], format='list_of_edges')
        sage: bispecial_vertices(G)
        [6]
        sage: G = DiGraph([(6,5), (7,6), (8,6)], format='list_of_edges')
        sage: bispecial_vertices(G)
        []

    """
    return [v for v in G.vertices() if G.in_degree(v) > 1 and G.out_degree(v) > 1]

def reduce_left_special_vertices(G, merge_function):
    r"""
    Merge all left special vertices with its in-neighbor(s) ``u`` using
    ``merge_function(u,v)`` to create the new vertex.

    INPUT:

    - ``G`` -- digraph
    - ``merge_function`` -- function taking two vertices as input and
      returning a new vertex

    OUTPUT:

    a digraph

    EXAMPLES::

        sage: from slabbe.graph import reduce_left_special_vertices
        sage: edges = [(0,1),(1,2),(2,3),(3,4),(4,0),(2,5),(5,6),(6,7),(7,0)]
        sage: edges = [(str(u),str(v)) for (u,v) in edges]
        sage: G = DiGraph(edges, format='list_of_edges')
        sage: merge_function = lambda u,v:u+v
        sage: GG = reduce_left_special_vertices(G, merge_function)
        sage: sorted((a,b) for (a,b,_) in GG.edges())
        [('2', '3'),
         ('2', '5'),
         ('3', '401'),
         ('401', '2'),
         ('5', '6'),
         ('6', '701'),
         ('701', '2')]

    It is idempotent::

        sage: GGG = reduce_left_special_vertices(GG, merge_function)
        sage: GGG == GG
        True

    """
    from copy import copy
    GG = copy(G)
    GG.allow_loops(True)
    #GG.allow_multiple_edges(True)

    v = get_left_special_vertex(GG)
    while v is not None:
        for u in GG.neighbors_in(v):
            u_v = merge_function(u, v)
            GG.add_edges((s, u_v) for s in GG.neighbors_in(u))
            if GG.out_degree(u) == 1:
                GG.delete_vertex(u)
            GG.add_edges((u_v, t) for t in GG.neighbors_out(v))
        GG.delete_vertex(v)
        v = get_left_special_vertex(GG)
    return GG

def reduce_right_special_vertices(G, merge_function):
    r"""
    Merge all right special vertices with its in-neighbor(s) ``u`` using
    ``merge_function(u,v)`` to create the new vertex.

    INPUT:

    - ``G`` -- digraph
    - ``merge_function`` -- function taking two vertices as input and
      returning a new vertex

    OUTPUT:

    a digraph

    EXAMPLES::

        sage: from slabbe.graph import reduce_right_special_vertices
        sage: edges = [(0,1),(1,2),(2,3),(3,4),(4,0),(2,5),(5,6),(6,7),(7,0)]
        sage: edges = [(str(u),str(v)) for (u,v) in edges]
        sage: G = DiGraph(edges, format='list_of_edges')
        sage: merge_function = lambda u,v:u+v
        sage: GG = reduce_right_special_vertices(G, merge_function)
        sage: sorted((a,b) for (a,b,_) in GG.edges())
        [('0', '123'),
         ('0', '125'),
         ('123', '4'),
         ('125', '6'),
         ('4', '0'),
         ('6', '7'),
         ('7', '0')]

    It is idempotent::

        sage: GGG = reduce_right_special_vertices(GG, merge_function)
        sage: GGG == GG
        True

    """
    from copy import copy
    GG = copy(G)
    GG.allow_loops(True)
    #GG.allow_multiple_edges(True)

    v = get_right_special_vertex(GG)
    while v is not None:
        for w in GG.neighbors_out(v):
            v_w = merge_function(v, w)
            GG.add_edges((v_w, s) for s in GG.neighbors_out(w))
            if GG.in_degree(w) == 1:
                GG.delete_vertex(w)
            GG.add_edges((t, v_w) for t in GG.neighbors_in(v))
        GG.delete_vertex(v)
        v = get_right_special_vertex(GG)
    return GG

def reduce_bispecial_vertices(G, merge_function, filter=None):
    r"""
    Merge all bispecial vertices with its in-neighbor(s) ``u`` using
    ``merge_function(u,v)`` to create the new vertex. Only edges such that
    ``filter(u,v)`` is True are kept.

    INPUT:

    - ``G`` -- digraph
    - ``merge_function`` -- function taking two vertices as input and
      returning a new vertex
    - ``filter`` -- function from pair of vertices to boolean (default:``None``),
      Only creates edges ``(u,v)`` such that ``filter(u,v) is True`` are kept.
      If ``None``, then ``filter = lambda a,b:True`` is used.

    OUTPUT:

    a digraph

    EXAMPLES::

        sage: from slabbe.graph import reduce_left_special_vertices
        sage: from slabbe.graph import reduce_bispecial_vertices
        sage: edges = [(0,1),(1,2),(2,3),(3,4),(4,0),(2,5),(5,6),(6,7),(7,0)]
        sage: edges = [(str(u),str(v)) for (u,v) in edges]
        sage: G = DiGraph(edges, format='list_of_edges')
        sage: merge_function = lambda u,v:u+v
        sage: GG = reduce_left_special_vertices(G, merge_function)
        sage: GGG = reduce_bispecial_vertices(GG, merge_function)
        sage: sorted((a,b) for (a,b,_) in GGG.edges())
        [('3', '4012'),
         ('4012', '3'),
         ('4012', '5'),
         ('5', '6'),
         ('6', '7012'),
         ('7012', '3'),
         ('7012', '5')]

    It is idempotent::

        sage: GGGG = reduce_bispecial_vertices(GGG, merge_function)
        sage: GGGG == GGG
        True

    """
    from copy import copy
    GG = copy(G)
    GG.allow_loops(True)
    #GG.allow_multiple_edges(True)

    if filter is None:
        filter = lambda a,b:True

    GG_bispecials = bispecial_vertices(GG)
    for v in GG_bispecials:
        if v not in GG:
            # v was deleted earlier in the process
            continue
        for u in GG.neighbors_in(v):
            u_v = merge_function(u, v)
            GG.add_edges((s, u_v) for s in GG.neighbors_in(u))
            if GG.out_degree(u) == 1:
                GG.delete_vertex(u)
            GG.add_edges((u_v, t) for t in GG.neighbors_out(v) if filter(u_v,t))
        GG.delete_vertex(v)
    return GG




# -*- coding: utf-8 -*-
r"""
Kolakoski Word

The classical Kolakoski sequence [K65]_ was first studied by Oldenburger
[O39]_, where it appears as the unique solution to the problem of a
trajectory on the alphabet `\{1,2\}` which is identical to its exponent
trajectory.

See http://en.wikipedia.org/wiki/Kolakoski_sequence

It uses cython implementation inspired from the 10 lines of C code written
by Dominique Bernardi and shared during Sage Days 28 in Orsay, France, in
January 2011. The alphabet must be ``(1,2)``.

EXAMPLES::

    sage: from slabbe import KolakoskiWord
    sage: K = KolakoskiWord()
    sage: K
    word: 1221121221221121122121121221121121221221...

The cython implementation is much faster than the python one::

    sage: K = KolakoskiWord()
    sage: K[10^6]      # takes 0.02 seconds
    2
    sage: K = words.KolakoskiWord()
    sage: K[10^6]      # not tested : takes too long
    2

TESTS:

We make sure both implementation correspond for the prefix of
length 100::

    sage: Kb = KolakoskiWord()
    sage: Kp = words.KolakoskiWord()
    sage: Kp[:100] == Kb[:100]
    True

REFERENCES:

.. [K65] William Kolakoski, proposal 5304, American Mathematical Monthly
   72 (1965), 674; for a partial solution, see "Self Generating Runs,"
   by Necdet Üçoluk, Amer. Math. Mon. 73 (1966), 681-2.

.. [O39] R. Oldenburger, Exponent trajectories in dynamical systems,
   Trans. Amer. Math. Soc. 46 (1939), 453–466.

AUTHORS:

- Sébastien Labbé (February 2011) - Added a Cython implementation
  which was easily translated from the 10 lines of C code written
  by Dominique Bernardi and shared during Sage Days 28 in Orsay.
  Needs review at #13346 since too long time...
"""
#*****************************************************************************
#       Copyright (C) 2011-2014 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from __future__ import absolute_import, print_function
from sage.combinat.words.infinite_word import InfiniteWord_class
from .kolakoski_word_pyx import WordDatatype_Kolakoski
 
class KolakoskiWord(WordDatatype_Kolakoski, InfiniteWord_class):
    def __init__(self, parent=None):
        r"""
        Constructor. See documentation of WordDatatype_Kolakoski for more
        details.

        EXAMPLES::

            sage: from slabbe import KolakoskiWord
            sage: K = KolakoskiWord()
            sage: K
            word: 1221121221221121122121121221121121221221...

        TESTS:

        Pickle is supported::

            sage: K = KolakoskiWord()
            sage: loads(dumps(K))
            word: 1221121221221121122121121221121121221221...
        """
        if parent is None:
            from sage.combinat.words.words import Words
            parent = Words([1,2])
        WordDatatype_Kolakoski.__init__(self, parent)




# -*- coding: utf-8 -*-
r"""
The code to construct the partitions in [Lab2019]_.

REFERENCES:

.. [Lab2019] S. Labbé.  Markov partitions for toral
    `\mathbb{Z}^2`-rotations featuring Jeandel-Rao Wang shift and model
    sets, https://arxiv.org/abs/1903.06137, April 2020 (v3).

.. [Lab2018] S. Labbé. A self-similar aperiodic set of 19 Wang
    tiles. Geom. Dedicata, 2018.
    https://doi.org/10.1007/s10711-018-0384-8.

EXAMPLES:

The partition associated to Jeandel-Rao Wang shift::

    sage: from slabbe.arXiv_1903_06137 import jeandel_rao_wang_shift_partition
    sage: P0 = jeandel_rao_wang_shift_partition()
    sage: P0
    Polyhedron partition of 24 atoms with 11 letters

The partition associated to the self-similar Wang shift `\Omega_{\mathcal{U}}`::

    sage: from slabbe.arXiv_1903_06137 import self_similar_19_atoms_partition
    sage: PU = self_similar_19_atoms_partition()
    sage: PU
    Polyhedron partition of 19 atoms with 19 letters

"""
#*****************************************************************************
#       Copyright (C) 2019-2020 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from sage.rings.polynomial.polynomial_ring import polygen
from sage.rings.rational_field import QQ
from sage.rings.number_field.number_field import NumberField
from sage.geometry.polyhedron.constructor import Polyhedron

from slabbe import PolyhedronPartition

def jeandel_rao_wang_shift_partition(backend=None):
    r"""
    This construct the polygon partition associated to Jeandel-Rao
    tilings introduced in [Lab2019]_.

    INPUT:

    - ``backend`` -- string, polyhedron backend

    EXAMPLES::

        sage: from slabbe.arXiv_1903_06137 import jeandel_rao_wang_shift_partition
        sage: P0 = jeandel_rao_wang_shift_partition()
        sage: P0.is_pairwise_disjoint()
        True
        sage: P0.volume()
        4*phi + 1

    The volume is consistent with::

        sage: z = polygen(QQ, 'z')
        sage: K = NumberField(z**2-z-1, 'phi', embedding=RR(1.6))
        sage: phi = K.gen()
        sage: phi * (phi + 3)
        4*phi + 1

    """
    # the golden mean
    z = polygen(QQ, 'z')
    K = NumberField(z**2-z-1, 'phi', embedding=QQ(1.6))
    phi = K.gen()

    # x and y coordinates
    xcoords = [0, phi**-2, phi**-1, 1, phi]
    ycoords = [0, 1, 2, phi+1, phi+2, phi+3]

    # the vertices
    A = [(x,0) for x in xcoords]
    B = [(x,1) for x in xcoords]
    C = [(x,2) for x in xcoords]
    D = [(x,phi+1) for x in xcoords]
    E = [(x,phi+2) for x in xcoords]
    F = [(x,phi+3) for x in xcoords]

    # atoms corresponding to Jeandel-Rao tile numbers
    L = [
        (0, (A[0], A[2], B[2])), 
        (0, (A[2], A[3], B[3])),
        (0, (A[3], A[4], B[4])),
        (1, (A[0], B[0], B[2])),
        (1, (A[2], B[2], B[3])),
        (1, (A[3], B[3], B[4])),
        (2, (C[0], D[0], E[2])),
        (3, (B[3], C[3], E[4], C[4])),
        (4, (C[0], E[2], E[3])),
        (4, (E[2], E[3], F[3])),
        (5, (D[0], E[0], E[2])),
        (5, (E[0], E[1], F[1])),
        (5, (E[1], E[2], F[3])),
        (6, (E[0], F[0], F[1])),
        (6, (E[1], F[1], F[3])),
        (6, (E[3], F[3], F[4])),
        (7, (D[3], E[3], E[4])),
        (7, (E[3], E[4], F[4])),
        (7, (B[0], C[0], E[3])),
        (8, (B[2], C[2], E[4])),
        (9, (B[0], B[2], C[2])),
        (9, (B[2], B[3], C[3])),
        (9, (B[3], B[4], C[4])),
        (10, (B[0], D[3], E[3])),
        ]
    L = [(key, Polyhedron(vertices, base_ring=K, backend=backend)) 
         for (key,vertices) in L]
    return PolyhedronPartition(L)

def self_similar_19_atoms_partition(backend=None):
    r"""
    This construct the polygon partition introduced in [Lab2019]_
    associated to the self-similar 19 Wang tiles [Lab2018]_.

    INPUT:

    - ``backend`` -- string, polyhedron backend

    EXAMPLES::

        sage: from slabbe.arXiv_1903_06137 import self_similar_19_atoms_partition
        sage: PU = self_similar_19_atoms_partition()
        sage: PU.is_pairwise_disjoint()
        True
        sage: PU.volume()
        1

    """
    # the golden mean
    z = polygen(QQ, 'z')
    K = NumberField(z**2-z-1, 'phi', embedding=QQ(1.6))
    phi = K.gen()

    # the partition vertices
    L = [
        (0, [(phi - 1, -2*phi + 4), (phi - 1, phi - 1), (-2*phi + 4, phi - 1)]),
        (1,
        [(phi - 1, -2*phi + 4),
        (phi - 1, 1),
        (1, 1),
        (-2*phi + 4, phi - 1),
        (1, phi - 1)]),
        (2, [(0, 1), (-phi + 2, phi - 1), (2*phi - 3, phi - 1)]),
        (3, [(0, 1), (0, phi - 1), (2*phi - 3, phi - 1)]),
        (4, [(phi - 1, phi - 1), (-phi + 2, 1), (phi - 1, -2*phi + 4)]),
        (5, [(phi - 1, 1), (-phi + 2, 1), (phi - 1, -2*phi + 4)]),
        (6, [(0, 1), (-phi + 2, 1), (-phi + 2, phi - 1)]),
        (7, [(-phi + 2, 1), (-phi + 2, phi - 1), (phi - 1, phi - 1)]),
        (8, [(1, 0), (phi - 1, 0), (phi - 1, -phi + 2)]),
        (9, [(-2*phi + 4, phi - 1), (phi - 1, phi - 1), (1, -phi + 2), (1, 0)]),
        (10, [(1, -phi + 2), (-2*phi + 4, phi - 1), (1, phi - 1)]),
        (11, [(1, 0), (phi - 1, -phi + 2), (phi - 1, phi - 1)]),
        (12, [(-phi + 2, -phi + 2), (-phi + 2, phi - 1), (2*phi - 3, phi - 1)]),
        (13,
        [(-phi + 2, phi - 1),
        (phi - 1, -phi + 2),
        (-phi + 2, -phi + 2),
        (phi - 1, 0)]),
        (14,
        [(2*phi - 3, phi - 1), (0, phi - 1), (-phi + 2, -phi + 2), (-phi + 2, 0)]),
        (15, [(phi - 1, 0), (-phi + 2, 0), (-phi + 2, -phi + 2)]),
        (16, [(0, 0), (-phi + 2, 0), (0, -phi + 2)]),
        (17, [(0, -phi + 2), (-phi + 2, 0), (0, phi - 1)]),
        (18, [(phi - 1, -phi + 2), (-phi + 2, phi - 1), (phi - 1, phi - 1)])
        ]

    L = [(key, Polyhedron(vertices, base_ring=K, backend=backend)) 
         for (key,vertices) in L]
    return PolyhedronPartition(L)




# -*- coding: utf-8 -*-
r"""
Matrix functions

EXAMPLES::

    ...

TODO:

    - Discrete geometry code should use projection_matrix from here

"""
#*****************************************************************************
#       Copyright (C) 2016 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from sage.matrix.constructor import matrix
import heapq

def projection_matrix(dim_from=3, dim_to=2):
    r"""
    Return a projection matrix from R^d to R^l.

    INPUT:

    - ``dim_from` -- integer (default: ``3``)
    - ``dim_to` -- integer (default: ``2``)

    OUTPUT:

        matrix

    EXAMPLES::

        sage: from slabbe.matrices import projection_matrix
        sage: projection_matrix(3,2)
        [-0.866025403784439  0.866025403784439  0.000000000000000]
        [-0.500000000000000 -0.500000000000000   1.00000000000000]
        sage: projection_matrix(2,3)
        [-0.577350269189626 -0.333333333333333]
        [ 0.577350269189626 -0.333333333333333]
        [ 0.000000000000000  0.666666666666667]
    """
    from math import sqrt
    from sage.rings.real_mpfr import RR
    sqrt3 = sqrt(3)
    if dim_from == 3 and dim_to == 2:
        return matrix(2,[-sqrt3,sqrt3,0,-1,-1,2],ring=RR)/2
    elif dim_from == 2 and dim_to == 3:
        return matrix(3,[-sqrt3,-1,sqrt3,-1,0,2],ring=RR)/3
    elif dim_from == 4 and dim_to == 2:
        return matrix(2,[-sqrt3,sqrt3,0,1,-1,-1,2,0],ring=RR)/2
    elif dim_from == 4 and dim_to == 3:
        sqrt2 = sqrt(2)
        return matrix([(1,-1,0,0), 
                       (0,0,1,-1),
                       (-1/sqrt2,-1/sqrt2,1/sqrt2,1/sqrt2)],
                       ring=RR)
    else:
        s = "for input dim_from={} and dim_to={}"
        raise NotImplementedError(s.format(dim_from, dim_to))

M3to2 = projection_matrix(3, 2)
M2to3 = projection_matrix(2, 3)
M4to2 = projection_matrix(4, 2)
M4to3 = projection_matrix(4, 3)

def perron_right_eigenvector(M):
    r"""
    EXAMPLES::

        sage: from slabbe.matrices import perron_right_eigenvector
        sage: m = matrix(2,[-11,14,-26,29])
        sage: perron_right_eigenvector(m)    # abs tol 0.0000001
        (15.0, (0.35, 0.65))
    """
    import numpy
    from sage.rings.real_mpfr import RR
    from sage.rings.all import CC
    from sage.modules.free_module_element import vector
    eig, vec = numpy.linalg.eig(M)
    index = abs(eig).argmax()
    rightv = vec.transpose()[index]
    if eig[index].imag == 0:
        eig_sage = RR(eig[index].real)
        vec_sage = vector(a.real for a in rightv)
    else:
        eig_sage = CC(eig[index])
        vec_sage = vector(CC, rightv)
    return eig_sage, vec_sage/sum(vec_sage)

def is_positive(M):
    r"""
    EXAMPLES::

        sage: from slabbe.matrices import is_positive
        sage: m = matrix(4, range(16))
        sage: is_positive(m)
        False
        sage: m = matrix(4, range(1,17))
        sage: is_positive(m)
        True
    """
    return all(a > 0 for a in M.list())

def is_nonnegative(M):
    r"""
    EXAMPLES::

        sage: from slabbe.matrices import is_nonnegative
        sage: m = matrix(4, range(-8,8))
        sage: is_nonnegative(m)
        False
        sage: m = matrix(4, range(16))
        sage: is_nonnegative(m)
        True
    """
    return all(a >= 0 for a in M.list())

def is_primitive(M):
    r"""
    EXAMPLES::

        sage: from slabbe.matrices import is_primitive
        sage: m = matrix(2, [0,1,1,1])
        sage: is_primitive(m)
        True
        sage: m = matrix(2, [1,1,0,1])
        sage: is_primitive(m)
        False
    """
    if not is_nonnegative(M):
        return False
    power = M
    order = 1
    dim = M.nrows()
    max_order = (dim-1)**2 + 1
    while True:
        l = power.list()
        if len(l) == 0:
            return False
        try:
            l.index(0)
        except ValueError:
            return True
        if order > max_order:
            return False
        power *= power
        order += order
def is_pisot(M):
    r"""
    EXAMPLES::

        sage: from slabbe.matrices import is_pisot
        sage: M = matrix(2,[1,1,0,1])
        sage: is_pisot(M)
        False

    ::

        sage: M = matrix(2,[0,1,1,1])
        sage: is_pisot(M)
        True
    """
    eig_module = map(abs, M.eigenvalues())
    largest, second_largest = heapq.nlargest(2, eig_module)
    return second_largest < 1

def conjugate_matrix_Z(M):
    r"""
    Return the conjugate matrix Z as defined in [1].

    EXAMPLES::

        sage: from slabbe.matrices import conjugate_matrix_Z
        sage: M = matrix(2, [11,29,14,-1])
        sage: conjugate_matrix_Z(M)       # abs tol 1e-8
        [11.674409930010482  27.69820597163912]
        [14.349386111618157  -1.67440993001048]
        sage: conjugate_matrix_Z(M)^2     # abs tol 1e-8
        [533.7440993001048 276.9820597163913]
        [143.4938611161816 400.2559006998952]

    ::

        sage: M = matrix(2, [-11,14,-26,29])
        sage: conjugate_matrix_Z(M)     # abs tol 1e-8
        [ 7.200000000000004  4.199999999999998]
        [ 7.799999999999995 10.800000000000002]
        sage: conjugate_matrix_Z(M) * 5     # abs tol 1e-8
        [ 36.00000000000002 20.999999999999993]
        [ 38.99999999999998 54.000000000000014]

    ::

        sage: M = matrix(2, [-11,26,-14,29]) / 15
        sage: conjugate_matrix_Z(M)     # abs tol 1e-8
        [ 0.5999999999999999  0.3999999999999999]
        [0.39999999999999986  0.5999999999999999]

    REFERENCES:

    [1] Labbé, Jean-Philippe, et Sébastien Labbé. « A Perron theorem for matrices
    with negative entries and applications to Coxeter groups ». arXiv:1511.04975
    [math], 16 novembre 2015. http://arxiv.org/abs/1511.04975.
    """
    from sage.matrix.special import identity_matrix
    eig, v = perron_right_eigenvector(M)
    v /= sum(v)
    nrows = M.nrows()
    UNtop = matrix([1]*nrows)
    v = matrix.column(v)
    id = identity_matrix(nrows)
    Z = eig*v*UNtop + (id - v*UNtop)*M
    return Z

def recurrence_matrix(coeffs):
    r"""
    Return the recurrence matrix of a relation, for example:

    INPUT: 

    - ``coeffs`` -- list of integers, for example if 
      R(n) = R(n-1) + 2 R(n-2) + 3R(n-3) + 4R(n-4) + 5R(n-5)
      then coeff must be [1,2,3,4,5]

    EXAMPLES::

        sage: from slabbe.matrices import recurrence_matrix
        sage: recurrence_matrix([1,2,3,4,5])
        [1 2 3 4 5]
        [1 0 0 0 0]
        [0 1 0 0 0]
        [0 0 1 0 0]
        [0 0 0 1 0]
    """
    poly = [-a for a in reversed(coeffs)]
    poly.append(1)
    return matrix.companion(poly, format='top')

def spectrum(M):
    r"""
    EXAMPLES::

        sage: from slabbe.matrices import spectrum, recurrence_matrix
        sage: M = recurrence_matrix([1,2,3,4,5])
        sage: spectrum(M)
        2.576021761956651?
    """
    return max(map(abs, M.eigenvalues()))

def column_norm_ratio(M, p=1):
    r"""
    Return the maximum of the ratio of the norm of two columns.

    INPUT:

    - ``p`` - default: 2 - ``p`` can be a real number greater than 1,
      infinity (``oo`` or ``Infinity``), or a symbolic expression.

      - `p=1`: the taxicab (Manhattan) norm
      - `p=2`: the usual Euclidean norm (the default)
      - `p=\infty`: the maximum entry (in absolute value)

    EXAMPLES::

        sage: from slabbe.matrices import column_norm_ratio
        sage: M = matrix(3, range(9))
        sage: column_norm_ratio(M)
        5/3

    """
    L = [column.norm(p) for column in M.columns()]
    m = min(L)
    if m == 0:
        return Infinity
    else:
        return max(L) / m

def map_coefficients_to_variable_index(M, x):
    r"""
    INPUT:

    - ``M`` -- matrix
    - ``x`` -- string, variable

    EXAMPLES::

        sage: from slabbe.matrices import map_coefficients_to_variable_index
        sage: M = matrix(2, range(4))
        sage: map_coefficients_to_variable_index(M, 's')
        [s_0 s_1]
        [s_2 s_3]
        sage: latex(_)
        \left(\begin{array}{rr}
        s_{0} & s_{1} \\
        s_{2} & s_{3}
        \end{array}\right)
    """
    from sage.calculus.var import var
    L = [var('{}_{}'.format(x,a)) for a in M.list()]
    return matrix(M.nrows(), L)

def Minkowski_embedding_without_sqrt2(self, B=None, prec=None):
    r"""
    This method is a modification of the ``Minkowski_embedding`` method of
    NumberField in sage (without sqrt2).

    EXAMPLES::

        sage: from slabbe.matrices import Minkowski_embedding_without_sqrt2
        sage: F.<alpha> = NumberField(x^3+2)
        sage: F.minkowski_embedding()
        [ 1.00000000000000 -1.25992104989487  1.58740105196820]
        [ 1.41421356237309 0.890898718140339 -1.12246204830937]
        [0.000000000000000  1.54308184421705  1.94416129723967]
        sage: Minkowski_embedding_without_sqrt2(F)
        [  1.00000000000000  -1.25992104989487   1.58740105196820]
        [  1.00000000000000  0.629960524947437 -0.793700525984099]
        [ 0.000000000000000   1.09112363597172   1.37472963699860]
        sage: Minkowski_embedding_without_sqrt2(F, [1, alpha+2, alpha^2-alpha])
        [ 1.00000000000000 0.740078950105127  2.84732210186307]
        [ 1.00000000000000  2.62996052494744 -1.42366105093154]
        [0.000000000000000  1.09112363597172 0.283606001026881]
        sage: Minkowski_embedding_without_sqrt2(F) * (alpha + 2).vector().column()
        [0.740078950105127]
        [ 2.62996052494744]
        [ 1.09112363597172]

    Tribo::

        sage: F.<beta> = NumberField(x^3-x^2-x-1)
        sage: F.minkowski_embedding()
        [  1.00000000000000   1.83928675521416   3.38297576790624]
        [  1.41421356237309 -0.593465355971987 -0.270804762516626]
        [ 0.000000000000000  0.857424571985895 -0.719625086862932]
        sage: Minkowski_embedding_without_sqrt2(F)
        [  1.00000000000000   1.83928675521416   3.38297576790624]
        [  1.00000000000000 -0.419643377607080 -0.191487883953119]
        [ 0.000000000000000  0.606290729207199 -0.508851778832738]

    Comprendre le problème de norme::

        sage: norme = lambda v:abs(v[0]) * (v[1]^2 + v[2]^2)
        sage: F.<beta> = NumberField(x^3-x^2-x-1)
        sage: M = Minkowski_embedding_without_sqrt2(F)
        sage: norme(M*vector((1,0,0)))
        1.00000000000000
        sage: norme(M*vector((1,0,-1)))
        4.00000000000000

    """
    r,s = self.signature()
    places = self.places(prec=prec)

    if B is None:
        B = [self.gen(0)**i for i in range(self.degree())]

    rows = []
    for i in range(r):
        rows.append([places[i](b) for b in B])
    for i in range(s):
        row_real = []
        row_imag = []
        for b in B:
            z = places[r+i](b)
            row_real.append(z.real())
            row_imag.append(z.imag())
        rows.append(row_real)
        rows.append(row_imag)

    from sage.matrix.constructor import matrix
    return matrix(rows)

def Minkowski_projection_pair(self, B=None, prec=None):
    r"""
    Return the projections to the expanding and contracting spaces.

    OUTPUT:

    - tuple (A, B) of matrices

    EXAMPLES::

        sage: from slabbe.matrices import Minkowski_projection_pair
        sage: F.<alpha> = NumberField(x^3+2)
        sage: Minkowski_projection_pair(F)
        (
        [  1.00000000000000  -1.25992104989487   1.58740105196820]
        [  1.00000000000000  0.629960524947437 -0.793700525984099]
        [ 0.000000000000000   1.09112363597172   1.37472963699860], []
        )
        sage: Minkowski_projection_pair(F, [1, alpha+2, alpha^2-alpha])
        (
        [ 1.00000000000000 0.740078950105127  2.84732210186307]
        [ 1.00000000000000  2.62996052494744 -1.42366105093154]
        [0.000000000000000  1.09112363597172 0.283606001026881], []
        )

    Tribo::

        sage: F.<beta> = NumberField(x^3-x^2-x-1)
        sage: Minkowski_projection_pair(F)
        (
        [1.000000000000000000000000000000 1.839286755214161132551852564671
        3.382975767906237494122708536521],
        [  1.00000000000000 -0.419643377607080 -0.191487883953119]
        [ 0.000000000000000  0.606290729207199 -0.508851778832738]
        )

    """
    r,s = self.signature()
    places = self.places(prec=prec)
    beta = self.gen()

    if B is None:
        B = [beta**i for i in range(self.degree())]

    rows_expanding = []
    rows_contracting = []

    for i in range(r):
        place = places[i]
        row = [place(b) for b in B]
        norm = place(beta).abs()
        if norm < 1:
            rows_contracting.append(row)
        elif norm > 1:
            rows_expanding.append(row)
        else:
            raise NotImplementedError

    for i in range(s):
        place = places[r+i]
        row_real = []
        row_imag = []
        for b in B:
            z = place(b)
            row_real.append(z.real())
            row_imag.append(z.imag())
        norm = place(beta).abs()
        if norm < 1:
            rows_contracting.append(row_real)
            rows_contracting.append(row_imag)
        elif norm > 1:
            rows_expanding.append(row_real)
            rows_expanding.append(row_imag)
        else:
            raise NotImplementedError

    from sage.matrix.constructor import matrix
    return (matrix(len(rows_expanding), self.degree(), rows_expanding),
            matrix(len(rows_contracting), self.degree(), rows_contracting))

def rauzy_projection(M, beta=None, prec=53):
    r"""
    Returns a projection matrix of the canonical basis using the Minkowski
    embedding associated to the left eigenvector of the given eigenvalue.

    INPUT:

    - ``beta`` - a real element of ``QQbar`` of degree >= 2 (default:
      ``None``).  The eigenvalue used for the projection.  It must be an
      eigenvalue of ``M``.  The one used by default is the maximal
      eigenvalue of ``M`` (usually a Pisot number), but matrices of order
      larger than 3 letters other interesting choices are sometimes
      possible.

    - ``prec`` - integer (default: ``53``), the number of bits used in the
      floating point representations of the coordinates.

    OUTPUT:

        matrix

    EXAMPLES:

    Fibonacci::

        sage: from slabbe.matrices import rauzy_projection
        sage: m = matrix(2,(1,1,1,0))
        sage: m
        [1 1]
        [1 0]
        sage: rauzy_projection(m)
        [ 1.000000000000000000000000000000 -1.618033988749894848204586834366]
        [ 1.000000000000000000000000000000 0.6180339887498948482045868343656]

    Tribonacci::

        sage: m = matrix(3, [1,1,1, 1,0,0, 0,1,0])
        sage: rauzy_projection(m)
        [  1.00000000000000  0.839286755214161  0.543689012692076]
        [  1.00000000000000  -1.41964337760708 -0.771844506346038]
        [ 0.000000000000000  0.606290729207199  -1.11514250803994]
        sage: matrix(2,(0,1,0, 0,0,-1))*rauzy_projection(m)
        [  1.00000000000000  -1.41964337760708 -0.771844506346038]
        [ 0.000000000000000 -0.606290729207199   1.11514250803994]

    which corresponds to the Rauzy fractal projection coded by Timo::

        sage: s = WordMorphism('1->12,2->13,3->1')
        sage: s.rauzy_fractal_projection()
        {'1': (1.00000000000000, 0.000000000000000),
         '2': (-1.41964337760708, -0.606290729207199),
         '3': (-0.771844506346038, 1.11514250803994)}

    TESTS::

        sage: t = WordMorphism('1->12,2->3,3->45,4->5,5->6,6->7,7->8,8->1')
        sage: m = matrix(t)
        sage: rauzy_projection(m).T
        [  1.00000000000000   1.00000000000000  0.000000000000000]
        [ 0.324717957244746  -1.66235897862237  0.562279512062301]
        [ 0.430159709001947  0.784920145499027  -1.30714127868205]
        [ 0.245122333753307   1.87743883312335  0.744861766619744]
        [ 0.324717957244746  -1.66235897862237  0.562279512062301]
        [ 0.430159709001947  0.784920145499027  -1.30714127868205]
        [ 0.569840290998053  0.215079854500973   1.30714127868205]
        [ 0.754877666246693 -0.877438833123346 -0.744861766619744]
        sage: t.rauzy_fractal_projection()
        {'1': (1.00000000000000, 0.000000000000000),
         '2': (-1.66235897862237, -0.562279512062301),
         '3': (0.784920145499027, 1.30714127868205),
         '4': (1.87743883312335, -0.744861766619744),
         '5': (-1.66235897862237, -0.562279512062301),
         '6': (0.784920145499027, 1.30714127868205),
         '7': (0.215079854500973, -1.30714127868205),
         '8': (-0.877438833123346, 0.744861766619744)}

    ::

        sage: E = t.incidence_matrix().eigenvalues()
        sage: x = [x for x in E if -0.8 < x < -0.7][0]
        sage: x
        -0.7548776662466928?
        sage: rauzy_projection(m, beta=x).T
        [  1.00000000000000   1.00000000000000  0.000000000000000]
        [ -1.75487766624669 -0.122561166876654  0.744861766619744]
        [  1.32471795724475 -0.662358978622373  0.562279512062301]
        [ -4.07959562349144 -0.460202188254281  0.182582254557443]
        [  3.07959562349144 -0.539797811745719 -0.182582254557443]
        [ -2.32471795724475 -0.337641021377627 -0.562279512062301]
        [  1.75487766624669  0.122561166876654 -0.744861766619744]
        [ -1.32471795724475  0.662358978622373 -0.562279512062301]
        sage: t.rauzy_fractal_projection(eig=x)
        {'1': (1.00000000000000, 0.000000000000000),
         '2': (-0.122561166876654, -0.744861766619744),
         '3': (-0.662358978622373, -0.562279512062301),
         '4': (-0.460202188254281, -0.182582254557443),
         '5': (-0.539797811745719, 0.182582254557443),
         '6': (-0.337641021377627, 0.562279512062301),
         '7': (0.122561166876654, 0.744861766619744),
         '8': (0.662358978622373, 0.562279512062301)}

    AUTHORS:

     - Timo Jolivet (2012-06-16) -- for substitutions in Sage
     - Sébastien Labbé (2018-03-08) -- for matrices, using Minkowski
       embedding
    """
    # Eigenvalue
    if beta is None:
        beta = max(M.eigenvalues(), key=abs)

    # One possibility is to do:
    # K,elt,hom = beta.as_number_field_element()

    # Left eigenvector vb in the number field Q(beta)
    from sage.rings.number_field.number_field import NumberField
    K = NumberField(beta.minpoly(), 'b')
    vb = (M-K.gen()).kernel().basis()[0]

    return Minkowski_embedding_without_sqrt2(K, vb)
    #return K.Minkowski_embedding(vb)

